//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.



// ************** Example ROOT script to test the program ****************
// Draws a set of histograms from a tree produced by the program
// In order to run the scripts place them in lxevent_root program subdirectory
// (if they are not already there).
// In ROOT shell just type .x lxjet_test_execute.C to run.



// -------------------- ROOT headers ----------------------
#include "TFile.h"
#include "TTree.h"
#include "TClassTable.h"
#include "TSystem.h"
#include "TROOT.h"
#include "TH1D.h"
#include "TLorentzVector.h"
#include "TStyle.h"
#include "TCanvas.h"
#include "Rtypes.h"
// --------------------------------------------------------

#include <iostream>
#include <vector>
#include "lxevent.h"



using namespace std;


// calculates azimutal angle between two vectors
double phiaz(const TLorentzVector & v1, const TLorentzVector & v2) {
 double cosphi = (v1(0) * v2(0) + v1(1) * v2(1)) 
      / TMath::Sqrt( TMath::Max( 1e-20, (v1(0)*v1(0) + v1(1)*v1(1)) * (v2(0)*v2(0) + v2(1)*v2(1)) ));  
 cosphi = TMath::Max(-1., TMath::Min(1., cosphi));
 return TMath::ACos(cosphi); }


  

// the main function
void lxjet_plot(char *alias_, char *path_) {



// --------------------- constants ----------------------
const double M_PI_ =  TMath::Pi();



// --------------------- Load ROOT trees ---------------------


Long64_t nevent; // number of stored events

LXEVENT *ev = new LXEVENT();
TTree *tree_;

TFile *file_;

char treename[400];
char filename[400];
char *tree_name = "tree";
char *root = ".root";

 tree_ = NULL;
 file_ = NULL;

  strcpy(filename,path_);
  strcat(filename,alias_);
  strcat(filename,root);
  strcpy(treename,alias_);
  strcat(treename,tree_name);
  file_ = new TFile(filename);
  tree_ = (TTree*)file_->Get(treename); 

 if (tree_ == NULL) cout << "ERROR: tree was not set. " << endl; 
  else cout << "The tree " << treename << " is read. Number of entries " << tree_->GetEntries() << endl;

tree_->SetBranchAddress("events",&ev);
nevent = tree_->GetEntries();

// ---------------------------------------------------------
// ------------------- Define histograms -------------------
// ---------------------------------------------------------


// ___________________ azimutal decorrelations ____________

TH1D *hist_phi;

Int_t nbinphi = 24; // number of bins
Double_t phimin = 0;
Double_t phimax = M_PI_;

hist_phi = new TH1D("hist_phi","azimuthal decorrelations between the leading and the softest jet",nbinphi,phimin,phimax);

// __________________ unbalanced pT _____________________

TH1D *hist_DpT;

Int_t nbinkT = 24;
Double_t kTmin = 0.;
Double_t kTmax = 50.;

hist_DpT = new TH1D("hist_DpT","unbalanced pT",nbinkT,kTmin,kTmax);


// _________________ x asymmetry ________________________

TH1D *hist_xas;

Int_t nbinxas = 24;
Double_t xasmin = 0.;
Double_t xasmax = 1.;

hist_xas = new TH1D("hist_xas","x asymmetry",nbinxas,xasmin,xasmax);


// _________________ leading jet pT _____________________

TH1D *hist_pT1;

Int_t nbinpT1 = 180;
Double_t pTmin1 = 20;
Double_t pTmax1 = 200;

hist_pT1 = new TH1D("hist_pT1","pT of the leading jet",nbinpT1,pTmin1,pTmax1);


// _________________ softest jet pT _____________________

TH1D *hist_pT3;

Int_t nbinpT3 = 180;
Double_t pTmin3 = 20;
Double_t pTmax3 = 200;

hist_pT3 = new TH1D("hist_pT3","pT of the softest jet",nbinpT3,pTmin3,pTmax3);


// _______________ hardest jet rapidity __________________

TH1D *hist_rap1;

Int_t nbinrap1 = 24;
Double_t rapmin1 = -5.;
Double_t rapmax1 = 5.;

hist_rap1 = new TH1D("hist_rap1","rapidity of the hardest jet",nbinrap1,rapmin1,rapmax1);


// _______________ softest jet rapidity __________________

TH1D *hist_rap3;

Int_t nbinrap3 = 24;
Double_t rapmin3 = -5.;
Double_t rapmax3 = 5.;

hist_rap3 = new TH1D("hist_rap3","rapidity of the hardest jet",nbinrap3,rapmin3,rapmax3);


// _______________ partons inside jets __________________

TH1D *hist_jets;

Int_t nbinjet = 5;
Double_t jetmin = 0;
Double_t jetmax = 5;

hist_jets = new TH1D("hist_jets","partons contributing in the hardest jet",nbinjet,jetmin,jetmax);

// -------------------------------------------------------------------
// ------------------ fill the histograms ----------------------------
// -------------------------------------------------------------------

cout << "Histogramming..." << endl;

TLorentzVector v1, v3;
Double_t wt, phi, kT, xA, xB, xas, En, rap1, rap3, pT1, pT3;
particles parton;

const Double_t ptcut = 0.; // additional pT cut that may be applied


Double_t binw, norm, totwt;

totwt = 0;

// set the energy stored in the tree
tree_->SetBranchAddress("CM_energy",&En);

// obtain the number of jets
tree_->GetEntry(1);
Int_t Njet = ev->get_out_states().size(); 

// event loop
for (Long64_t i=1;i<=nevent;++i) {    

tree_->GetEntry(i);

v1 = ev->get_out_states(0); // hardest jet momentum
v3 = ev->get_out_states(Njet-1); // softest jet momentum

rap1 = v1.Rapidity();
rap3 = v3.Rapidity();

pT1 = v1.Pt();
pT3 = v3.Pt();

if ( pT3 > ptcut ) {

kT = ev->get_in_states(0).Pt();

xA = 2*ev->get_in_states(0).T()/En;
xB = 2*ev->get_in_states(1).T()/En;
xas = TMath::Abs(xA-xB)/(xA+xB);

phi = phiaz(v1,v3); 

parton = ev->get_out_flavours(0);

wt =  ev->get_weight();
totwt += wt;

hist_phi->Fill(phi,wt);
hist_DpT->Fill(kT,wt);
hist_xas->Fill(xas,wt);
hist_pT1->Fill(pT1,wt);
hist_pT3->Fill(pT3,wt);
hist_rap1->Fill(rap1,wt);
hist_rap3->Fill(rap3,wt);
if ( parton == G ) hist_jets->Fill(1,wt);
 else { 
   if ( (parton == Q) || (parton == q) ) hist_jets->Fill(2,wt);
     else hist_jets->Fill(3,wt); } 

} 
}


cout << "Done." << endl;

hist_phi->Sumw2();
hist_DpT->Sumw2();
hist_xas->Sumw2();
hist_pT1->Sumw2();
hist_pT3->Sumw2();
hist_rap1->Sumw2();
hist_rap3->Sumw2();

// ---------- normalize the histograms -------------------
// (this should be automatized)

binw = (phimax-phimin)/(double)nbinphi;
norm = ev->get_normalization()/binw/totwt;
hist_phi->Scale(norm);


binw = (kTmax-kTmin)/(double)nbinkT;
norm = ev->get_normalization()/binw/totwt;
hist_DpT->Scale(norm);


binw = (xasmax-xasmin)/(double)nbinxas;
norm = ev->get_normalization()/binw/totwt;
hist_xas->Scale(norm);

binw = (pTmax1-pTmin1)/(double)nbinpT1;
norm = ev->get_normalization()/binw/totwt;
hist_pT1->Scale(norm);


binw = (pTmax3-pTmin3)/(double)nbinpT3;
norm = ev->get_normalization()/binw/totwt;
hist_pT3->Scale(norm);


binw = (rapmax1-rapmin1)/(double)nbinrap1;
norm = ev->get_normalization()/binw/totwt;
hist_rap1->Scale(norm);


binw = (rapmax3-rapmin3)/(double)nbinrap3;
norm = ev->get_normalization()/binw/totwt;
hist_rap3->Scale(norm);


norm = ev->get_normalization()/totwt;
hist_jets->Scale(norm);

// ----------------------------- draw plots -----------------------------------

Double_t vals[] = { hist_jets->GetBinContent(2), hist_jets->GetBinContent(3), hist_jets->GetBinContent(4)};
Int_t nvals = 3;
Int_t colors[] = {2,4,5};

TPie *pie = new TPie("pie1","contribution of partons in the leading jet",nvals,vals,colors);
pie->SetEntryLabel(0,"gluons");
pie->SetEntryLabel(1,"quarks");
pie->SetEntryLabel(2,"anti-quarks");
pie->SetLabelsOffset(-.15);

pie->SetLabelFormat("%perc");

TCanvas *canv = new TCanvas("canv","Plots",1200,800);
canv->Divide(3,3);
canv->cd(1);
hist_phi->Draw("E1hist");
canv->cd(2);
hist_DpT->Draw("E1hist");
canv->cd(3);
hist_xas->Draw("E1hist");
canv->cd(4);
hist_pT1->Draw("E1hist");
canv->cd(5);
hist_pT3->Draw("E1hist");
canv->cd(6);
hist_rap1->Draw("E1hist");
canv->cd(7);
hist_rap3->Draw("E1hist");
//canv->cd(8);
//hist_jets->Draw("hist");
canv->cd(9);
pie->Draw("r NOL");
TLegend *pieleg = pie->MakeLegend();
pieleg->SetY1(0.9); pieleg->SetY2(0.7);
pieleg->SetX1(0.);pieleg->SetX2(0.3);



}//******************************************************************************************









