/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  \author Wojtek Slominski, Jagiellonian Univ., Physics Dept.
  \date 2005--2012
  \copyright GNU General Public License
*/
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
_____________________________________________________________*/

#ifndef _FIASCO_H_
#define _FIASCO_H_

#ifdef _EMSG_H_
  #error Either fiasco.h or emsg.h allowed -- not both!
#endif

#include <iostream>
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <string>
#include <sstream>
#include <cstdarg>
#include <stdexcept>
#include <exception>

// using namespace std;

/// Exception class
// ooooooooooooooooooooooooooooooo
class Fiasco : public std::exception {

  std::string msg;
  char buf[1024];
  
public:
  // =============================
  Fiasco(const std::string& s) {
    // msg = "S: "+s;
    msg = s;
  }
  
  // =============================
  Fiasco(const char* fmt, va_list apt) {
    if(strchr(fmt,'%')) {
      // va_start(apt,fmt);
      vsprintf(buf,fmt,apt);
      msg = buf;
    }
    else msg = fmt;
    
    va_end(apt);
	}
  
  // =============================
  Fiasco(const char* fmt,...) {
    if(strchr(fmt,'%')) {
      // char buf[1024]="F: ";
      va_list apt;
      va_start(apt,fmt);
      vsprintf(buf,fmt,apt);
      va_end(apt);
      msg = buf;
    }
    else msg = fmt;
	}
  
  // =============================
  ~Fiasco() throw() {}
	
  // =============================
	const char* what() const throw() {
    // return ("FIASCO: "+msg).c_str();
    return msg.c_str();
  }
  
};

/// ostringstream class that throws on destruction
// -o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o
class Veto : public std::ostringstream {
public:
  ~Veto() throw(std::string) {throw str();}
};

// --- global function for backward compatibility
// ==============================================
void require(bool cond, const char *fmt, ...);


#define START_XCODE try {

#define CATCH_ALL \
  catch(const logic_error& x) {std::cerr <<"logic_error: " << x.what() << endl; } \
  catch(const runtime_error& x) {std::cerr <<"runtime_error: " << x.what() << endl; } \
  catch(const Fiasco& f) {std::cerr << f.what() << endl;}\
  catch(std::string msg) {std::cerr << msg << endl;} \
  catch(const char* msg) {std::cerr << msg << endl;} \
  catch(Veto::failure e) {std::cerr << e.what() << endl;}

#define END_XCODE } CATCH_ALL \
  std::cerr << "\n*** FATAL error - program terminated." << endl; \
  exit(1);

#endif
