/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  \author Wojtek Slominski, Jagiellonian Univ., Physics Dept.
  \date 2005--2013
  \copyright GNU General Public License
*/
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
_____________________________________________________________*/

#ifndef TYPES_H_
#define TYPES_H_

// #include <cfloat>
#include <limits>
#include <cmath>
#include <vector>

/** @addtogroup gr_mathut
 *
 *  @{
 */

#if USE_LONG_DOUBLE
  typedef long double real_type;
  // const real_type real_type_max=DBL_MAX;
#else
  typedef double real_type;
  // const real_type real_type_max=LDBL_MAX;
#endif

const real_type real_type_max = std::numeric_limits<real_type>::max();
const int real_type_digits = std::numeric_limits<real_type>::digits10;
const real_type real_type_acc = pow(0.1,real_type_digits);

typedef real_type*  RealPtr_t;
typedef RealPtr_t* RealPtr2_t; //--- 2 dim array
typedef const real_type*  ConstRealPtr_t;
typedef const RealPtr_t* ConstRealPtr2_t;
typedef real_type (*RealFcn)(real_type);
typedef real_type (*RealFcn1)(real_type);
typedef real_type (*RealFcn2)(real_type, real_type);

/// Pure virtual base class for 1-arg \c real_type function
// -o-o-o-o-o-o-o-o-o-o-o-o
struct RealFunction1_t {
  virtual real_type operator()(real_type x) = 0;
};

typedef std::vector<real_type> real_vec_t;

///@}

#endif
