//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef lxevent_h
#define lxevent_h


#include "TObject.h"
#include "TLorentzVector.h"

#include <vector>

using namespace std;


//--------------------------------------------------------------------------------------------------
//------------------------------------------ LXEVENT -----------------------------------------------
//--------------------------------------------------------------------------------------------------

#ifndef common_h
enum particles {
    G = 0,
    Rg = 1, // off-shell gluon (reggeized gluon)
    Q = 2,
    Qb = 3,
    q = 4,
    qb = 5 };
#endif

class LXEVENT: public TObject {
  
public:
  
  void set_in_states(const TLorentzVector & p1, const TLorentzVector & p2) {
    in_states[0] = p1;
    in_states[1] = p2;  } // set the vectors for initial states
    
  TLorentzVector get_in_states(int i) const { return in_states.at(i); } // obtain the ith vector for initial states
  
  void set_out_states(const vector <TLorentzVector> & out_states_) { 
    out_states.clear();
    for (int i=0; i<out_states_.size(); ++i) out_states.push_back(out_states_[i]);  } // set the vectors for final states
  
  TLorentzVector get_out_states(int i) const { return out_states.at(i); } // obtain the ith vector for final states
  
  vector<TLorentzVector> get_out_states() const { return out_states; } // obtain the ith vector for final states
  
  void set_flavours( const vector<particles> & particles_) {
    in_flavours[0] = particles_[0];
    in_flavours[1] = particles_[1];
    out_flavours.clear();
    for ( int i=2; i<particles_.size();++i) out_flavours.push_back(particles_[i]);} // set the flavours for partons
  
  particles get_in_flavours(int i) const { return in_flavours.at(i); } // get the flavour of ith initial state parton
  
  particles get_out_flavours(int i) const { return out_flavours.at(i); } // get the flavour of ith final state parton
  
  void set_weight( double weight_) { weight = weight_; } // set the weight of the event
  
  void set_normalization( double integral_) { integral = integral_; } // set the normalization integral
  
  Double_t get_weight() const { return weight; } // obtain the weight of the event
  
  Double_t get_normalization() const { return integral; } // obtain the normalization integral
  
  LXEVENT() { in_states.resize(2); in_flavours.resize(2); } 
  
  ~LXEVENT() {}
  
  
  
private:
  
  
  vector<TLorentzVector> in_states;  // vector of initial state four-momenta
  vector<TLorentzVector> out_states; // vector of final-state state four-momenta
  
  vector<particles> in_flavours;  // vector of initial state parton types
  vector<particles> out_flavours; // vector of final-state state parton types
  
  Double_t weight; // MC weight of the event
    
  Double_t integral; // the normalization
  
  TLorentzVector vtmp; // auxiliary
  
    ClassDef(LXEVENT,1);
  
};



#endif