//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef me_2_h
#define me_2_h

#include "me_N.h"



//--------------------------------------------------------------------------------------------------
//------------------------------------------- ME_2 --------------------------------------------------
//--------------------------------------------------------------------------------------------------

/// Derived class for matrix elements for two particles in the final state. Matrix element is defined
/// for single configuration of flavours.
class ME_2:public ME_N {
 
 public:
 
  /// Realization of virtual function; it calls function given by a pointer
 void calculate_me2(const CONF_N & conf) { 
   fill_kinematics(conf);
   (this->*ME_2::calculate_pt)(); }
   
   /// Realization of virtual function; it calls function given by a pointer
 void calculate_me2(const CONF_N & conf, double V[]) { 
   fill_kinematics(conf);
   (this->*ME_2::calculate_hel_pt)(V); }  

 /// This constructor assumes passing whole state space table with all subprocesses
 /// and the index of the actual sub-process.
 ME_2(const vector< vector<particles> > & state_space, int nproc);
 
 /// This constructor assumes passing only sub-process vector.
 ME_2(const vector<particles> & subproc);

 virtual ~ME_2() {  }

 private:
   
 /// Auxiliary vectors for final state momenta 
 TLorentzVector k3, k4;
 
 /// Realization of virtual function
 void fill_kinematics(const CONF_N & conf);
 
 /// Set the pointer to a function calculating actual ME
 void set_pointers();
 
 /// Prepare polarization vectors if recursion relation is used
 void prepare_polvecs();
   
 /// Amplitude squared for G*G->QQb and helicity LRL; these here and below are not color avareged.
 double me2_RgGQQb_LRL();
 /// Amplitude squared for G*G->QQb and helicity LLR
 double me2_RgGQQb_LLR();
 
 /// Amplitude squared for g*g->gg and helicity LLL
 double me2_RgGGG_LLL();
 /// Amplitude squared for g*g->gg and helicity LLR
 double me2_RgGGG_LLR();
 /// Amplitude squared for g*g->gg and helicity LRL
 double me2_RgGGG_LRL(); 
 
 /// Calculates amplitude squared for G*G->GG summed over helicities and averaged over color
 void calculate_me2_RgGGG();
 
 /// Calculates amplitude squared for G*G->QQb summed over helicities and averaged over color
 void calculate_me2_RgGQQb();
 
 /// Calculates amplitude squared for G*Q->QG (or G*Qb->QbG) summed over helicities and averaged over color
 void calculate_me2_RgQQG();
 
 /// Calculates amplitude squared for G*G->GG averaged over color
 void calculate_me2_RgGGG(double V[]);
 
 /// Calculates amplitude squared for G*G->QQb averaged over color
 void calculate_me2_RgGQQb(double V[]);
 
 /// Calculates amplitude squared for G*Q->QG (or G*Qb->QbG) averaged over color
 void calculate_me2_RgQQG(double V[]);
 
 /// A pointer for function calculating the ME for actual process
 void (ME_2::*calculate_pt)();
 
 /// A pointer for function calculating the helicity ME for actual process
 void (ME_2::*calculate_hel_pt)(double V[]);

 /// Auxiliary
 TLorentzVector vtmp;
 
};




#endif
