//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef procspace_h
#define procspace_h

#include <vector>
#include <string>

#include "common.h"

#include "TLorentzVector.h"



//--------------------------------------------------------------------------------------------------
//---------------------------------------- PROCSPACE -----------------------------------------------
//--------------------------------------------------------------------------------------------------


/// Class that implements space of all subprocesses contributing to the given process.
class PROCSPACE {
  
public:
  
  /// Obtain the state space table
  vector< vector<particles> > get_state_space() const { return state_space; }
  
  /// Obtain a subprocess of the state space table
  vector<particles> get_state_space(int i) const { return state_space.at(i); }
  
  PROCSPACE( const vector< vector<particles> > & state_space_ ) { state_space = state_space_; }
  
  /// In this constructor one can use some predefined subprocess tables once one gives a number of jets
  PROCSPACE(int Njet_ );
  
private:

 /// The vector of vectors corresponding to a single subprocess.
 /// Thus, it is a vector consisting of {In_1, In_2, Out_1, ..., Out_N},
 /// where In_i and Out_j are of "particles" enum type
 vector< vector<particles> > state_space;
  
};



	 
//************************************************************************************************
//*********************************** STATE SPACE TABLES *****************************************
//************************************************************************************************


/// The space of all partonic sub-processes for hadron-hadron -> 2 jets with single reggeon
					  
const particles state_space_HHto2j[][4] = {
					   Rg,G,G,G,
					   Rg,G,Q,Qb,
					   Rg,Q,Q,G,
					   Rg,Qb,Qb,G,
					  };					  
					  

/// The space of all partonic sub-processes for hadron-hadron -> 3 jets with single reggeon
 
const particles state_space_HHto3j[][5] = {  
					  Rg,G,G,G,G,
					  Rg,G,Q,Qb,G,
					  Rg,Q,Q,G,G, 
					  Rg,Q,Q,Q,Qb,
					  Rg,Q,Q,q,qb,
					  Rg,Qb,Qb,G,G,
					  Rg,Qb,Qb,Qb,Q,
					  Rg,Qb,Qb,qb,q 
 };



#endif