//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef xsec_h
#define xsec_h



#include "common.h"
#include "conf_N.h"
#include "pdfprod.h"
#include "me_N.h"
#include "lxjet.h"

#include "TFoamIntegrand.h"


using namespace std;


//--------------------------------------------------------------------------------------------------
//-------------------------------------------- XSEC ------------------------------------------------
//--------------------------------------------------------------------------------------------------


/// The class that calculates the cross section using objects for matrix elements,
/// PDFs, and kinematics. It is the class that gives probability density for
/// the FOAM generator.
class XSEC:public TFoamIntegrand {

public:


  /// The realization of virtual function representing density distribution
  double Density(int ndim, double V[]);
  
  /// Get the generated event number according to a specific state space table
  int get_event_number() const { return event_no; }
  
  /// Obtain the indicator if the internal cuts were satisfied
  bool get_retain_event() const { return retain_event; }
  
  /// Obtain the sign of the weight;
  signs get_wt_sign() const { return wt_sign; }
  
  /// Obtain the weight;
  double get_wt() const { return weight; }
  
  /// Constructor use user-class object and the present class can change it
  XSEC(LXJET* &lxjet);
         
  virtual ~XSEC();
   
   
private:
  
  /// The value of the weight
  double weight;
  
  /// The sign of the weight
  signs wt_sign;
  
  /// A vector of pointers to different objects of ME_N class
  vector<ME_N*> mels;
  
  /// A vector of pointers to different objects of PDF class
  vector<PDFPROD*> pdfs;
  
  /// The generated number of event according to specific table of events
  int event_no;
  
  /// The pointer to kinematic handling CONF_N class
  CONF_N *conf;
  
  /// A pointer to a lxjet object that was used to create the present class object
  LXJET * const lxjet_pt;
  
  /// The number of sub-processes contributin to the cross section
  int Ntotproc;
  
  /// Indicated if the internal cuts were satisfied
  bool retain_event;


};




#endif

