//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "test_forward_central_trijets.mod.h"


TFC3j_INIT::TFC3j_INIT() {

// ------ set MC parameters ------

  // Set the number of FOAM cells at initialization
 const int foam_Ncell = 10000; 
  // Set the number of FOAM events per cell
 const int foam_Nevcell = 10000;
 // for the true calculation the above parameters should be increased
 
  // Set the total statistics
 const long Nevmax = 1000000;
  // Set the number of events after which the events are saved
 const long Nevsav = 100000; 
  // Weighted =true, unweighted = false events
 const bool unweighted =  false;

// ----------------------------------------------------------------- 
 
  // CM energy
 const double Ecm7TeV = 7000;
 const double Ecm5TeV = 5000;
 
 // unintegrated PDF with improved BFKL evolution of Kwiecinski-Martin-Stasto
 // LxJet_BFKL_set is a path automatically defined in the preprocessor inside defconf.h
 PDF* p_BFKL = new PDF_grid(LxJet_BFKL_set,unintegrated);
 
 // declare objects of user-classes 
 // pass to the constructor alias of the run and CM energy 
 LXJET* test_fctrijets;
 test_fctrijets = new TEST_FCTRIJETS("test_trijets",Ecm7TeV,*p_BFKL);
 
 // register user-class; then it can be run by passing the alias 
 // as the program command-line parameter

 RUNS::register_user(test_fctrijets,Nevmax,foam_Ncell,foam_Nevcell,Nevsav,unweighted);
 
}




TEST_FCTRIJETS::TEST_FCTRIJETS(string alias_, double CMenergy_, PDF& hadron_A_) { 
 set_params(
    *alphaqcd, // predefined in USER_CLASS
    hadron_A_,  
    *collinear_proton, // predefined in USER_CLASS
    *trijets, // predefined in USER_CLASS         
    CMenergy_,           
    5, // Number of flavours                       
    alias_           
  );
}



bool TEST_FCTRIJETS::basic_cuts(const CONF_N & conf)
{
    
   // Fill the momenta (they are orginally in CM(pA,pB)
   event.clear();
   event.push_back(conf.get_k1());
   event.push_back(conf.get_k2());
   event.push_back(conf.get_ki(0));
   event.push_back(conf.get_ki(1));
   event.push_back(conf.get_ki(2));
      
   // define the jets
   jets = vector<TLorentzVector>(event.begin()+2,event.end());
      
   // set factorization and renormalization scales
   double mu=0.;
   // the total energy of the jets
   for (int i=0; i<3; i++) mu += jets[i].E();
   mu /= 2.; // scale variation factor
   set_mu_fact(mu);
   set_mu_ren(mu);
   
   // result in pico barns
   set_mult_fact(389379.323*1000.);
   
   // cut on phi-rapidity space corresponding to clustering algorithm with R=0.5
   if (phi_rap_cut(jets,0.5)) return reject();
   
   // cut on pT of the jets
   const double pTmin = 35.;
   for (vector<TLorentzVector>::iterator it = jets.begin();it!=jets.end();++it) if (it->Pt() < pTmin) return reject();
   
   // sort the jets with pT
   sort(jets.begin(),jets.end(),pT_sort());

   // calculate rapidities
   rap.clear();
   for (vector<TLorentzVector>::iterator it = jets.begin();it!=jets.end();++it) rap.push_back(rapidity(*it));
      
   // central-forward rapidity cuts
   const double rapc = 2.8; // central jets cut
   const double rapf1 = 3.2; // lower bound on forward jets
   const double rapf2 = 4.7; // upper bound on forward jets
   for (int i=0; i<2; i++)  if (abs(rap[i]) >= rapc) return reject();
    if (abs(rap[2]) <= rapf1 || abs(rap[2]) >= rapf2) return reject();
  
   // save trial events in a text file for visulaization
     //save_trial_events(1,10);
    
  return accept();

}

