//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include <sstream>

#include "common.h"
#include "polvec.h"

using namespace std;



//************************************************************************************************
//************************************* GLOBAL VARIABLES *****************************************
//************************************************************************************************


namespace LxJet {

double Ep;

double CM_energy;

double xA_min;

double xA_max;

double xB_min;

double xB_max;

double kT_min;

double kT_max;

int Nf;

string prompt;

ostream *out_stream;

}


//************************************************************************************************
//************************************* GLOBAL FUNCTIONS *****************************************
//************************************************************************************************


ostream & operator<<(ostream & monitor, particles state) {
  switch (state) {    
    case Q:
      monitor << "Q";
      break;
    case Qb:
      monitor << "Qb";  
      break;
    case G:
      monitor << "G";  
      break;
    case Rg:
      monitor << "RG";  
      break;
    case q:
      monitor << "q";  
      break;
    case qb:
      monitor << "qb";  
      break;
    default:
      monitor << state;
      break;
  }
    return monitor;
}



// __________________________________________________________________________________________________



ostream & operator<<(ostream & out_, const TLorentzVector & v) {
  out_ <<  v.X() << "   " << v.Y() << "   " << v.Z() << "   " << v.E();
  return out_; }



// __________________________________________________________________________________________________



template <class type>
ostream & operator<<(ostream & monitor, vector<type> & vv) {
    int size = vv.size();
    for (int i=0; i<size; i++) {
        monitor << vv.at(i) << "\t";
    } monitor << endl;
    return monitor;
}

template ostream & operator<<(ostream & monitor, vector<double> & vv);
template ostream & operator<<(ostream & monitor, vector<int> & vv);
template ostream & operator<<(ostream & monitor, vector<particles> & vv);
template ostream & operator<<(ostream & monitor, vector<helicity> & vv);
template ostream & operator<<(ostream & monitor, vector<TLorentzVector> & vv);
template ostream & operator<<(ostream & monitor, vector<POLVEC> & vv);
template ostream & operator<<(ostream & monitor, vector < vector< POLVEC > > & vv);



// __________________________________________________________________________________________________



void print_proc(ostream & out_, const vector<particles> & vv) {
    for (int i=0; i<2; i++) out_ << vv.at(i) << " ";
    out_ << "-> ";
    for (int i=2; i<vv.size(); i++) out_ << vv.at(i) << " "; out_ << endl;
}



// __________________________________________________________________________________________________



template <class type>
vector<type> vector_from_array(const type array[],int size) {
  vector<type> tmp;
  for (int i=0; i<size; i++) tmp.push_back(array[i]);
  return tmp;
}

template vector<particles> vector_from_array(const particles array[], int size);
template vector<helicity> vector_from_array(const helicity array[], int size);
template vector<int> vector_from_array(const int array[], int size);



// __________________________________________________________________________________________________



void print_3D_event(ostream & out_, const vector<TLorentzVector> & event) {
  for (int i = 0; i<2; i++) {
    out_ << setprecision(10) << -event[i].Pz() << "\t" << -event[i].Px() << "\t" << -event[i].Py() << "\t" 
	<< event[i].Pz()  << "\t" << event[i].Px()  << "\t" << event[i].Py() << endl;
  }
  for (int i = 2; i<event.size(); i++) {
    out_ << setprecision(10) << 0. << "\t" << 0. << "\t" << 0. << "\t" << event[i].Pz() 
     << "\t" << event[i].Px() << "\t" << event[i].Py() << endl;
  }
  out_ << "\n\n" << endl;
}




// __________________________________________________________________________________________________



void print_error(ostream & out_, const string & message) {
  out_ << LxJet::prompt << "ERROR: " << message << endl;
  out_ << LxJet::prompt << "Quitting..." << endl;
  exit(0);
}




// __________________________________________________________________________________________________



void print_message(ostream & out_, const string & message) {
  out_ << LxJet::prompt << message << endl;
}



// __________________________________________________________________________________________________



template <class numtype>
string operator<<(const string & str, numtype X) {
  return str+ static_cast< ostringstream* >( &(ostringstream() << X) )->str();}
  
template string  operator<<(const string & str, double X);
template string  operator<<(const string & str, int X);
template string  operator<<(const string & str, long X);

