//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "conf_2.h"



//--------------------------------------------------------------------------------------------------
//-------------------------------------- CONF_2 ----------------------------------------------------
//--------------------------------------------------------------------------------------------------


CONF_2::CONF_2() {
 	    
    Nfs = 2;
    ki.resize(Nfs);
}



// __________________________________________________________________________________________________



void CONF_2::generate(double V[]) {

  generate_basics(V);
    
    // generate direction of kT
    alph = 2.*M_PI*V[3];
    
    scale_fact *= 2.*M_PI;
    
    sinal = sin(alph);
    cosal = cos(alph);
    
    // generate angle theta
    theta1 = M_PI*V[4];
    
    scale_fact *= M_PI;
    
    sinth1 = sin(theta1);
    costh1 = cos(theta1);
    
    // generate angle phi
    phi1 = 2.*M_PI*V[5];
    
    scale_fact *= 2.*M_PI;
    
    sinphi1 = sin(phi1);
    cosphi1 = cos(phi1);

      
    // generate initial states in CM(pA,pB)
	
	// generate random parton origin (if such option is set)
        (this->*CONF_N::symmetrize_pt)();
	
	k1.SetXYZT(kT*cosal,kT*sinal,0.,0.);
        k1 = k1 + xA*p_in[0];
	
	k2 = xB*p_in[1];
    
	// polarization direction for off-shell gluon
	e1.SetXYZT(cosal,sinal,0.,0.);
	
	// CM(k1,k2)
	k1cm = k1; k2cm = k2;
		
	CM_frame(Lcm,k1,k2);
	
	k1cm *= Lcm;
	k2cm *= Lcm;

        //generate FS momenta

        S12 = (k1+k2).M2();

        Ei = sqrt(S12)/2.;
	

        ki[0].SetXYZT(Ei*sinth1*sinphi1,Ei*sinth1*cosphi1,Ei*costh1,Ei);
        ki[1].SetXYZT(-Ei*sinth1*sinphi1,-Ei*sinth1*cosphi1,-Ei*costh1,Ei);
	
	Lcm.Invert();
       
        for (int i=0; i<2; i++) {
	  ki[i] *= Lcm; }
	
	// ---------------- PS check ---------------------
	/*
	cout << "+++++ PS2 check +++++" << endl;
	cout << " momentum conservation:" << endl;
	cout << k1+k2-ki[0]-ki[1] << endl;
	cout << " onshellness:" << endl;
	cout << ki[0].M2() << "   " << ki[1].M2() << endl;
	*/
	// -----------------------------------------------
	  
	phase_space_fact = sinth1/32./M_PI/M_PI;  // 2-particle massless PS           
}

