//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "conf_3.h"


//--------------------------------------------------------------------------------------------------
//-------------------------------------- CONF_3 ----------------------------------------------------
//--------------------------------------------------------------------------------------------------


CONF_3::CONF_3() {
 	    
   Nfs = 3;
   ki.resize(Nfs);
}



// __________________________________________________________________________________________________



void CONF_3::generate(double V[]) {
  
  generate_basics(V);
            
    // generate an angle of kT
    alph = 2.*M_PI*V[8]; 

    scale_fact *= 2.*M_PI;
    
    cosal = cos(alph);
    sinal = sin(alph);
    
         
    // generate initial states in CM(pA,pB)
          
        // generate random parton origin (if such an option is set)
        (this->*CONF_N::symmetrize_pt)();
	
	k1.SetXYZT(kT*cosal,kT*sinal,0.,0.);
        k1 = k1 + xA*p_in[0];
	
	k2 = xB*p_in[1];
	
	// polarization direction for off-shell gluon
	e1.SetXYZT(cosal,sinal,0.,0.);
	
	
	// CM(k1,k2)
	
	k1cm = k1; k2cm = k2;
	CM_frame(Lcm,k1,k2);
	
	k1cm *= Lcm;
	k2cm *= Lcm;
	
	MM = pow2(k1cm.E()+k2cm.E()); // CM energy squared
		
        M = sqrt(MM);

	
	//generate FS momenta in CM(p,q)
	//let P=p+q; in this frame P=(M,0,0,0)
	
	mm23 = MM*V[3];
	
	scale_fact *= MM;
	
	m23 = sqrt(mm23);
	
	// now, in CM(2,3)
	
	E1s = (MM-mm23)/m23/2.;
	
	E3s = m23/2.;
	
	mm13 = 4.*E1s*E3s*V[4];
	
	scale_fact *= 4.*E1s*E3s;
	
	
	// back in CM(p,q)
	
	E1 = (MM - mm23)/M/2.;
	
	E2 = (MM - mm13)/M/2.;
	
	E3 = M - E1 - E2;
		
	
	costh = (MM-2.*M*(E1+E2)+2.*E1*E2)/E1/E2/2.;
	
	sinth = sqrt(1.-costh*costh);
	
	
	// define decay to be on XZ plane with k1 pointing to +X
	
	ki[0].SetXYZT(E1,0,0,E1);
	
	ki[1].SetXYZT(E2*costh,E2*sinth,0,E2);
	
	ki[2].SetXYZT(-E1-E2*costh,-E2*sinth,0,E3);
	
	// now we rotate the decay plane
		
	   // generate angle phi
           phi = 2.*M_PI*V[6];
    
           scale_fact *= 2.*M_PI;
    
           // generate angle phi1
           phi1 = 2.*M_PI*V[5];
    
           scale_fact *= 2.*M_PI;
    
           // generate angle theta
           theta = M_PI*V[7];
    
           scale_fact *= M_PI;
	
	for (int i=0; i<3; i++) {
	 rotate_Euler(ki[i],phi1,theta,phi); }
       
       // go back to CM(pA,pB)
         
       Lcm.Invert();
              
       for (int i=0; i<3; ++i)   ki[i] *= Lcm; 
       
       // ---------------- PS check ---------------------
	/*
	cout << "+++++ PS3 check +++++" << endl;
	cout << " momentum conservation:" << endl;
	cout << k1+k2-ki[0]-ki[1]-ki[2] << endl;
	cout << " onshellness:" << endl;
	cout << ki[0].M2() << "   " << ki[1].M2() << "   " << ki[2].M2() << "\n" << endl;
        */
	// -----------------------------------------------	
	
	
       phase_space_fact =  sin(theta)/MM/pow(2.*M_PI,5)/32.;  // 3-particle PS	   
}



// __________________________________________________________________________________________________



void CONF_3::rotate_Euler(TLorentzVector & v, double alpha, double beta, double gamma) {
  // re-coded the code of Wojtek Slominski
  // alpha --> [0, 2pi), beta --> [0,pi), gamma --> [0, 2pi)
  double cb = cos(beta);
  double sb = sin(beta);
  double ca=cos(alpha);
  double sa=sin(alpha);
  double cg=cos(gamma);
  double sg=sin(gamma);
  double x = (cg*ca - sg*cb*sa)*v.X() + (-sg*ca - cg*cb*sa)*v.Y() + sb*sa*v.Z();
  double y = (cg*sa + sg*cb*ca)*v.X() + (-sg*sa + cg*cb*ca)*v.Y() - sb*ca*v.Z();
  double z = sg*sb*v.X() + cg*sb*v.Y() + cb*v.Z();
  v.SetX(x); v.SetY(y); v.SetZ(z); }