//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "conf_N.h"


//--------------------------------------------------------------------------------------------------
//------------------------------------------ CONF_N ------------------------------------------------
//--------------------------------------------------------------------------------------------------


CONF_N::CONF_N():symmetrize_pt(&CONF_N::symmetrize),IdL() {
  
  if (LxJet::xA_max<LxJet::xA_min) print_error(*LxJet::out_stream,"in CONF_N::CONF_N: xA_max < xA_min");
  if (LxJet::xB_max<LxJet::xB_min) print_error(*LxJet::out_stream,"in CONF_N::CONF_N: xB_max < xB_min");
  if (LxJet::kT_max<LxJet::kT_min) print_error(*LxJet::out_stream,"in CONF_N::CONF_N: kT_max < kT_min");
  
   set_firm_scale_factors();   
   reconstruct_proton_moms();
   
   p_in.push_back(pA);
   p_in.push_back(pB);   
}



// __________________________________________________________________________________________________



void CONF_N::generate_basics(double V[]) {
  
  // This function has to be called first of all.
    reject = false;
    
    xA = LxJet::xA_min + (LxJet::xA_max-LxJet::xA_min)*V[0];
    
    xB = LxJet::xB_min + (LxJet::xB_max-LxJet::xB_min)*V[1];

    DkT = min(LxJet::kT_max,sqrt(xA*xB)*LxJet::CM_energy) - LxJet::kT_min;
    
    kT = LxJet::kT_min + DkT*V[2];    
       
    if (DkT >0. ) 
      scale_fact = firm_scale_fact*DkT;
    else 
      if (DkT==0.) scale_fact = firm_scale_fact;
       else reject = true;
}



// __________________________________________________________________________________________________



void CONF_N::set_firm_scale_factors() {
  
  firm_scale_fact = 1.;
  
  bool chk_xA = (LxJet::xA_min == LxJet::xA_max);
  bool chk_xB = (LxJet::xB_min == LxJet::xB_max);
  
  double D_xA = LxJet::xA_max - LxJet::xA_min;
  double D_xB = LxJet::xB_max - LxJet::xB_min;
  
     if ( !chk_xA ) firm_scale_fact *= D_xA;
     if ( !chk_xB ) firm_scale_fact *= D_xB; 
 }  
 
 

// __________________________________________________________________________________________________



void CONF_N::CM_frame(TLorentzRotation & L_, const TLorentzVector& p1, const TLorentzVector& p2) {
  vCM = (p1 + p2).BoostVector(); 
  pCM = p1; // will be along +Z
  // reset L matrix
  L_ = IdL;
  L_.Boost(vCM);
  L_.Invert();
  pCM *= L_;
  double theta = pCM.Theta();
  double phi = pCM.Phi();
  L_.RotateZ(-phi);
  L_.RotateY(-theta);
  L_.RotateZ(phi);
  
}
