/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  \author Wojtek Slominski, Jagiellonian Univ., Physics Dept.
  \date 2013
  \copyright GNU General Public License
*/
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
_____________________________________________________________*/

/*
  Polynomial interpolation
*/

#include <cmath>
#include "Interpolator.h"

/// Global function for 1-dim polynomial interpolation of arbitrary degree
//========================================================================
real_type LagrangeIpol(const real_type xVal[], const real_type fVal[], int deg, real_type x) {
  real_type y = 0, c;
  int i,j;
  for(i = 0; i <= deg; i++) {
    c = 1;
    for(j = 0; j <= deg; j++) if(j != i) c *= (x-xVal[j])/(xVal[i]-xVal[j]);
    y += fVal[i]*c;
  }
  return y;
}

//=========================================================================
real_type Interpolator_t::value(real_type x) {
  Locate(x);
  return LagrangeIpol(&(X->front())+Last_ix, V+Last_ix, deg, x);
}

//=========================================================================
real_type Interpolator_t::value(real_type *vals) {
  return LagrangeIpol(&(X->front())+Last_ix, vals+Last_ix, deg, Last_x);
}


// ----------  2-dim interpolation
// _______________________________________

//=========================================================================
real_type Interpolator2_t::value(const real_type *vals) {
  const real_type *LastX2ptr;
  LastX2ptr = &(X2->front())+Last_ix2;
  real_type zb[deg1+1];
  // --- Val[ix1][ix2] == V[ix1*Nx2+ix2]
  for(int k = 0; k <= deg1; k++) zb[k] = LagrangeIpol(LastX2ptr, vals+((Last_ix1+k)*Nx2+Last_ix2), deg2, Last_x2);
  return LagrangeIpol(&(X1->front())+Last_ix1, zb, deg1, Last_x1);
}
