/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  \author Wojtek Slominski, Jagiellonian Univ., Physics Dept.
  \date 2013
  \copyright GNU General Public License
*/
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
_____________________________________________________________*/

#include "vseq.h"

// #undef DBG_SHOW
// #undef DBG_HERE
// #define DBG_SHOW(a) cout <<"*** "<< __FILE__ <<"("<< __LINE__<<"): " << #a" = '" << (a) <<"'"<< endl;
// #define DBG_HERE(msg) cout <<"*** "<< __FILE__ <<"("<< __LINE__<<"): " << msg << endl; 
/// @cond NIEWAZNE
#undef SHOW
#define SHOW(a)
// #define SHOW(a) cout << #a" = " << (a) << endl;
/// @endcond 

// ==========================================
static bool GT(real_type a, real_type b) {return a > b;}

#define KLAPA throw Fiasco("%g outside the allowed range [%g, %g)", x, xL, xR)

// ==========================================
void VSeq_t::sort_check() {
  int N=size()-1;
  if(N < 1) throw "VSeq_t: at least two fixed points (endpoints) required.";
  isIncr = back() > front();
  if(isIncr) sort(begin(), end());
  else sort(rbegin(), rend());
  for(int i=0; i < N; i++) {
    if(at(i+1) == at(i)) throw "VSeq_t: values must be different.";
  }
  SetTolerance();
  OK = true;
}
    
// ==========================================
void VSeq_t::Fill(real_vec_t& fixed, real_type hmax) {
  clear();
  fi.clear();
  int N=fixed.size()-1;
  if(N < 1) throw "in VSeq_t::Fill: at least two fixed points (endpoints) required.";
  isIncr = hmax > 0;
  if(isIncr) sort(fixed.begin(), fixed.end());
  else sort(fixed.rbegin(), fixed.rend());
  for(int i=0; i < N; i++) {
    if(fixed[i+1] == fixed[i]) throw "in VSeq_t::Fill: fixed points must be different.";
    if(i) fi.push_back(size());
    int n = ceil((fixed[i+1] - fixed[i])/hmax);
    if(!n) n++;
    SHOW(n)
    real_type h = (fixed[i+1] - fixed[i])/n;
    SHOW(h)
    for(int j=0; j < n; j++) push_back(fixed[i] + j*h);
  }
  push_back(fixed[N]);
  SetTolerance();
  OK = true;
}
    
//===========================================
VSeq_t::const_iterator VSeq_t::Find(real_type x) const {
  SHOW(isIncr)
  if(!isIncr) return FindRev(x);
  real_type xL = at(0);
  int N = size()-1;
  real_type xR = at(N);
  SHOW(tol_abs_beg)
  SHOW(tol_abs_end)
  if(x < xL) {
    if(x < xL-tol_abs_beg) KLAPA;
    else return begin();
  }
  if(x >= xR) {
    // --- do not include right edge of the range
    if(x >= xR+tol_abs_end) KLAPA;
    else return end()-2;
  }
  return std::upper_bound(begin(), end(), x)-1;
}

//===========================================
VSeq_t::const_iterator VSeq_t::FindRev(real_type x) const {
  SHOW(x)
  SHOW(tol_abs_beg)
  SHOW(tol_abs_end)
  real_type xL = at(0);
  int N = size()-1;
  real_type xR = at(N);
  if(!GT(x, xR)) {
    if(!GT(x, xR-tol_abs_end)) KLAPA;
    else return end()-2;
  }
  if(GT(x, xL)) {
    if(GT(x, xL+tol_abs_beg)) KLAPA;
    else return begin();
  }
  return std::upper_bound(begin(), end(), x, GT)-1;
}

#undef KLAPA
#undef SHOW
