//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#define main_function_

#include <fstream>
#include <cmath>
#include <iomanip>
#include <vector>
#include <iostream>
#include <string>
#include <cstdio>
#include <ctime>
#include <getopt.h>

#include "common.h"
#include "xsec.h"
#include "userfun.h"
#include "modules.h"

#include "TSystem.h"
#include "TRandom.h"
#include "TRandom3.h"
#include "TFoamIntegrand.h"
#include "TFoam.h" 
#include "TLorentzVector.h"

#include <complex>

//************************************************************************************************
//********************************************* MAIN *********************************************
//************************************************************************************************



int main( int argc, char* argv[] )
{

  
// Used to display time statistics   
time_t startt, endt;
time (&startt);
int dtime, hours, minutes, seconds;


// --------------  options handling --------------------------------------------
LxJet::out_stream=NULL; // A pointers to potential writing mechanism, global variable
streambuf *sbuf=NULL; // A blind buffer to be written instead od cout
filebuf *fbuf=NULL; // A file buffer to write logfile
int opt; // option for getopt
opterr = 0;// resides in getopt lib.; dumps error message for wrong option
bool optyn = 0; // controls excluding options
bool optyn1 = 0; // controls excluding options
int foam_pers_opt = 0; // options for foam persistency; 0 = no persistency, 1 = write/read, 2 = overwrite
string optmes = "The options {-v,-s,-f} and {-I,-R} are exclusive within each group.";
string usage = string("\n  ")+string(LxJet_VERSION)
       +string("\n  Copyright (C) 2013 Piotr Kotko\n")
       +string("  This program is a free software and comes with ABSOLUTELY NO WARRANTY. \n")
       +string("  You may redistribute and modify it under GNU GPL v3 (or later),\n  see COPYING.txt file.\n\n")
       +string("Usage: ")+string(argv[0])+string("  [-OPTIONS] <user-class alias>\n")
       +string("  -h         prints this help message\n")
       +string("  -s         silent mode; no LxJet status is written to the screen or a file\n")
       +string("  -f<name>   writes current status only to the log file <name>\n")
       +string("  -v         default and can be skipped; prints the output to the screen\n")
       +string("  -I         writes the initialization foam to ROOT file (see FOAM manual)\n")
       +string("             the name of the file is <user-class alias>_foam.root\n")
       +string("             if the file exists, the foam from the file is used\n")
       +string("  -R         same as -I, but if the file exists, it is overwritten\n\n")
       +string("  The options {-v,-s,-f} and {-I,-R} are exclusive within each group.\n\n");

	       
while ((opt = getopt(argc, argv, "sf:hvIR")) != -1) {
 switch (opt) {
   case 'h':
    cout << usage << endl; 
    exit(0);
    break; 
   case 's':
    if (optyn==1) { cout << optmes << endl; exit(0);}
    optyn = 1;
    sbuf = new stringbuf;
    LxJet::out_stream = new ostream(sbuf);
    break;
   case 'f':
    if (optyn==1) { cout << optmes << endl; exit(0);}
    optyn = 1; 
    fbuf = new filebuf;
    fbuf->open(optarg,std::ios::out);
    LxJet::out_stream = new ostream(fbuf);
     break;
   case 'v':
     if (optyn==1) { cout << optmes << endl; exit(0);}
    optyn = 1;
    LxJet::out_stream = &cout;
    break;
   case 'I':
     if (optyn1==1) { cout << optmes << endl; exit(0);}
    optyn1 = 1;
    foam_pers_opt = 1;
    break;
   case 'R':
     if (optyn1==1) { cout << optmes << endl; exit(0);}
    optyn1 = 1;
    foam_pers_opt = 2;
    break; 
 
   default:
      cout << usage << endl;   exit(0);     }  }       

if ( argc == 2 && optind == 1) LxJet::out_stream = &cout; // if no option is given treat the same as -v
 else // if signle option is given different than -vsf add -v
  if ( argc == 3 && optind == 2 && optyn == 0) LxJet::out_stream = &cout; 
   else 
    if ( argc != (optind+1) || argc < 2 ) { cout << usage << endl; exit(0); }

//---------------------- end of options ---------------------------------------------------------------




// ---------------------------- LxJet initialization  ---------------------------------

LXJET *lxjet_user = NULL;

*LxJet::out_stream << scientific;

// Display greetings, version etc
*LxJet::out_stream << "\n\nStarting " << LxJet_VERSION << " ..." << endl;

string user_class_alias(argv[optind]);

USER_ENTRY &user_entry = RUNS::call_user(user_class_alias); // find the user-class to be used

lxjet_user = user_entry.get_pointer();


// --- the main MC loop parameters ---
long Nevmax = user_entry.get_Nevmax(); // the max number of accepted events 
long Nevsav = user_entry.get_Nevsav(); // the number of accepted events after which thay are saved

// --- the main FOAM parameters that should be set by the user these are default ---
int nCells = user_entry.get_foam_Ncell(); // number of foam cells at startup
int nEvCell = user_entry.get_foam_Nevcell(); // number of events per cell
bool unweighted_yn = user_entry.get_unweighted_yn(); // FOAM parameter -- weighted or unweighted events


*LxJet::out_stream << "Using user-class aliased as \"" << user_class_alias << "\"\n" << endl;

LxJet::prompt = user_class_alias+string("@")+string("LxJet> ");

// check if the user-class and basic parameters are initialized
if (lxjet_user == NULL) print_error(*LxJet::out_stream,"user-class was not initialized.");
if (!lxjet_user->is_initialized()) print_error(*LxJet::out_stream,"user-class parameters were not initialized.");

// check corectness of Nevmax and Nevsav parameters  
if (Nevmax<0 || Nevsav<0 || Nevsav>Nevmax) print_error(*LxJet::out_stream,(string)"wrong Nevmax or Nevsav parameters.");

//check if global kinematic variables are set
if (!lxjet_user->is_globalized()) 
print_error(*LxJet::out_stream,"Kinematic variables are not transfered \
  to the global LxJet namespace. The method LXJET::initialize_kinematic_range() has to be called."); 

// Create the density function
XSEC *Dens = new XSEC(lxjet_user);

// ------------------------------ end of LxJet Initialization -------------------------------------



// ------------------------------ initialization of FOAM ------------------------------------------

string root_file_name = user_class_alias+string("_foam.root");

TFoam *FoamX = NULL;
TRandom *PseRan    = new TRandom3();     // Create random number generator

// check if the file with cells exists
ifstream root_file_(root_file_name.c_str());
if (root_file_.good() && foam_pers_opt == 1) {
   TFile root_file(root_file_name.c_str());
   FoamX = (TFoam*)root_file.Get("FoamX");
    if (FoamX!=NULL) 
      print_message(*LxJet::out_stream,"FOAM cells succesfully read."); 
     else print_error(*LxJet::out_stream,"Coudn't read FOAM cells."); }
else {     

// FOAM related stuff and settings
  FoamX = new TFoam("FoamX");    // Create Simulator  
  FoamX->SetkDim(lxjet_user->get_Ndim());        // Number of dimensions
  FoamX->SetnCells(nCells);                // Number of Cells
  FoamX->SetnSampl(nEvCell);               // Number of MC events per cell in build-up
  FoamX->SetOptRej(unweighted_yn);         // =0, weighted events;  =1, wt=1 events
  FoamX->SetnBin(50);
  FoamX->SetEvPerBin(100);
  FoamX->SetChat(0);

  // One might want not to divide some of the dimensions...
  //FoamX->SetInhiDiv(8,1);
  
  
string init_info = (string("Initializing FOAM (")<<nCells)
                   +(string(" cells, ")<<nEvCell)+string(" events per cell)...");

*LxJet::out_stream  << LxJet::prompt << init_info << endl;  		   
		   
  FoamX->SetRho(Dens);
  FoamX->SetPseRan(PseRan);
  
  FoamX->Initialize(); // Initialize FOAM cells  
  
  if (foam_pers_opt > 0) {
    TFile root_file(root_file_name.c_str(),"RECREATE","foam cells");
    FoamX->Write("FoamX");
    root_file.Write();
    root_file.Close();  }
  
}

*LxJet::out_stream  << LxJet::prompt << "Initialization done." << endl;  

// ----------------------------- end of FOAM initialization ---------------------------------------




// ------------------------ create user-defined histograms or trees -------------------------------
lxjet_user->book();
// ------------------------------------------------------------------------------------------------

//FoamX->CheckAll(1);


// --------------------------- variables needed in MC loop ----------------------------------------
double *MCvect = new double[FoamX->GetTotDim()];// The vector generated in the MC run  
double MCwt;// MC weight
const int Nbadmax = 100; // the numer of maximal events that are NAN
long Nbad = 0;// the number of NAN events
long loop=1; // the main MC loop counter
long tot_acc_ev = 0; // the total number of accepted events
double tot_wt_acc = 0; // total accepted weight;
double MCeff; // = sigma/average_weight
double eps=0.0005; // epsilon for maxwt of FOAM
double maxwt; // maximal weight
double avwt; // average weight
double sigma; // standard deviation
double integral, integral1; // The MC integral
double integ_err, integ_err1; // The MC integral error
double integ_norm; // the normalization integral
double integ_norm_err; // the normalization integral error
bool stats_print_yn = false; // auxiliary; indicated if the statistics are to be printed
// ------------------------- end of definitions of variables ----------------------------------------




// -------------------------------- the main MC loop ------------------------------------------------


// a file to dump temorary commands of FOAM (or ROOT in general) user does not need
char dump_file[] = {"dump.txt"}; 

*LxJet::out_stream <<  LxJet::prompt << "Entering MC loop... " << "" << endl;


do {  
  
// Generate MC event
FoamX->MakeEvent();             

// Get MC vectors
FoamX->GetMCvect(MCvect);


// Get the weight
MCwt=FoamX->GetMCwt();

// A) if the weight is not NAN, proceed
if (!isnan(MCwt)) {
   
// pass the weight to the user class  
lxjet_user->set_weight(Dens->get_wt_sign()*MCwt); 
// pass also the number of the event from the space of the states
lxjet_user->set_event_number(Dens->get_event_number());

// if "basic_cuts" (implemented in the density function) or phase space cuts are satisfied, proceed
if (Dens->get_retain_event()) { 

// if the additional cuts are satisfied, fill histograms or trees
if (!lxjet_user->cuts()) { stats_print_yn = true; lxjet_user->fill(); ++tot_acc_ev; tot_wt_acc += MCwt; }  // Fill histograms or a tree if cuts are satisfied
 } 

// if the number of events that satisfy all the cuts is a multiplicity of
// Nevsav save to disc and print info
  if ( (tot_acc_ev % Nevsav == 0) && stats_print_yn ) {
   stats_print_yn = false; // avoid printing the same multiple times      
   // calculate time statistics    
   time (&endt);
   dtime = difftime (endt,startt);
   hours = dtime/3600;
   minutes = (dtime%3600)/60;
   seconds = (dtime%3600%60);
  
   // obtain weight parameters
   gSystem->RedirectOutput(dump_file,"w"); // --> trick to avoid printing foam garbage (temporary)
   FoamX->GetWtParams(eps,avwt,maxwt,sigma);
   gSystem->RedirectOutput(0);                   // <--
  
   // obtain the normalization integral and pass to user-class
   FoamX->GetIntNorm(integ_norm,integ_norm_err);
   FoamX->GetIntegMC(integral,integ_err);
   
   if (unweighted_yn) integral1 = integral*tot_acc_ev/loop;
    else integral1 = tot_wt_acc*integ_norm/loop;
   integ_err1 = integ_err*tot_acc_ev/loop;
   lxjet_user->set_integral(integral1);
   MCeff = sigma/avwt;
   
   *LxJet::out_stream << LxJet::prompt << tot_acc_ev << " of " << loop << " events saved (" 
	      << hours << "h " << minutes << "min " << seconds << "sec)" << endl;
   *LxJet::out_stream << LxJet::prompt << "Integral = " 
	      << integral1 << " +- " << integ_err1 << endl;
   *LxJet::out_stream << LxJet::prompt << "variance/<weight> = "<< MCeff << endl;
	      
  lxjet_user->save();
  
  }
  
} // if NAN write warning
else { Nbad++; if (Nbad>Nbadmax) print_error(*LxJet::out_stream,(string)"the number of NAN events exceeded "<<Nbadmax);
                   else print_message(*LxJet::out_stream,(string)"WARNING: encountered NAN event, in total "<<Nbad); }

++loop;
} while ( tot_acc_ev <  Nevmax);
// ---------------------------------- end of main MC loop -----------------------------------------------------------





// ------------------------------------------- finishing ------------------------------------------------------------
*LxJet::out_stream << endl << LxJet::prompt << "Job completed." << endl;
if (Nbad > 0) 
*LxJet::out_stream << endl << LxJet::prompt << "WARNING: number of NAN events : " << Nbad << "." << endl;

*LxJet::out_stream <<  LxJet::prompt << "Quitting..." << endl;

gSystem->Unlink(dump_file);
 
  delete [] MCvect;
  delete FoamX;
  delete PseRan;
  delete Dens;
  lxjet_user->close();
  delete lxjet_user;
  if (LxJet::out_stream != NULL && LxJet::out_stream != &cout) delete LxJet::out_stream;
  if (sbuf != NULL) delete sbuf;
  if (fbuf != NULL) { fbuf->close(); delete fbuf; }
    
}


