//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "me_2.h"


// uncomment if the recursion relations for gluons are to be used
// they are a bit slower than analytic formulae
//#define RECURSION_2


//--------------------------------------------------------------------------------------------------
//------------------------------------------ ME_2 --------------------------------------------------
//--------------------------------------------------------------------------------------------------




ME_2::ME_2(const vector< vector<particles> > & state_space, int nproc) : ME_N(2) {
  states = state_space.at(nproc);
  set_pointers(); }



// __________________________________________________________________________________________________



ME_2::ME_2(const vector<particles> & subproc) : ME_N(2)  {
  states = subproc;
  set_pointers();  }



// __________________________________________________________________________________________________



void ME_2::set_pointers() {
  
  // define auxiliary vectors of flavours
  particles RgGGG[] = {Rg,G,G,G};
  particles RgGQQb[] = {Rg,G,Q,Qb};
  particles RgQQG[] = {Rg,Q,Q,G};
  particles RgQbQbG[] = {Rg,Qb,Qb,G};
  
  if ( is_permutation(states,vector_from_array(RgGGG,4)) )
  { ME_2::calculate_pt = & ME_2::calculate_me2_RgGGG;
    ME_2::calculate_hel_pt = & ME_2::calculate_me2_RgGGG;
    prepare_polvecs();  }
  
  else if ( is_permutation(states,vector_from_array(RgGQQb,4)) )
  { ME_2::calculate_pt = & ME_2::calculate_me2_RgGQQb;
    ME_2::calculate_hel_pt = & ME_2::calculate_me2_RgGQQb; }
  
  else if ( is_permutation(states,vector_from_array(RgQQG,4)) )
  { ME_2::calculate_pt = & ME_2::calculate_me2_RgQQG;
    ME_2::calculate_hel_pt = & ME_2::calculate_me2_RgQQG;}
  
  else if ( is_permutation(states,vector_from_array(RgQbQbG,4)) )
  { ME_2::calculate_pt = & ME_2::calculate_me2_RgQQG;
    ME_2::calculate_hel_pt = & ME_2::calculate_me2_RgQQG;}
  
  else print_error(*LxJet::out_stream,"in ME_2::ME_2: process not implemented.");  }



// __________________________________________________________________________________________________



void ME_2::prepare_polvecs() { 
  
  
 // prepare helicites
 helicity  hel_LLL[] = {L,L,L};
 helicity  hel_LLR[] = {L,L,R};
 helicity  hel_LRL[] = {L,R,L};

 helicities.clear();
 helicities.push_back(vector_from_array(hel_LLL,3));
 helicities.push_back(vector_from_array(hel_LLR,3));
 helicities.push_back(vector_from_array(hel_LRL,3));
 
 // number of polarization vectors
 const int Np = 3;
 
 // prepare polarization vectors
 polvecs.clear();
 polvecs.resize(Np);
 momenta.clear();
 momenta.resize(Np);
 order.clear();
 order.resize(Np);

 // prepare color permutations
 color_ords = find_nonequiv_permutations(1,Np+1);
 
 // prepare array of the off-shell currents
    matrix_of_currents.clear();
    
     // construct trivial currents
     for (int i=0; i<Np; i++) {
      tmp_VJ.push_back(tmp_J); }
      
     matrix_of_currents.push_back(tmp_VJ); 
     tmp_VJ.clear();
         
     // construct the rest
     for (int i=1; i<=(Np-2); i++) {
       for (int j=0; j<(Np-i); j++) {
	  tmp_VJ.push_back(tmp_J);  } 
        matrix_of_currents.push_back(tmp_VJ); 
	tmp_VJ.clear(); }
}



// __________________________________________________________________________________________________



void ME_2::fill_kinematics(const CONF_N& conf) {
    assign(k1,conf.get_k1());
    assign(k2,conf.get_k2());
    assign(k3,conf.get_ki(0));
    assign(k4,conf.get_ki(1));
    assign(e1,conf.get_e1()); // 'polarization' of the off-shell gluon
    kTv = conf.get_kT(); // kT of the off-shell gluon
    assign(n,k1); // gauge vector
    n -= kTv*e1; 
}

 
// __________________________________________________________________________________________________
 
 
 
void ME_2::calculate_me2_RgGQQb() {
     
         me2 = me2_RgGQQb_LLR();      // sum over helicities
         me2 += me2_RgGQQb_LRL();   
	 me2 /= (2.*M_PI)*2.*8.*8.;   // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	 me2 *= 2.*LxJet::Nf;         // comes from flipped helicities and number of FS flavours
}
  


// __________________________________________________________________________________________________



void ME_2::calculate_me2_RgGQQb(double V[]) {
  
  int hel_dim = 2;  
  int hel_no = gen_int(V[0],hel_dim);
  
  switch (hel_no) {
    case 0:
         me2 = me2_RgGQQb_LLR();
	 break;
    case 1:
         me2 = me2_RgGQQb_LRL();   
	 break;
  }
	 me2 /= (2.*M_PI)*2.*8.*8.;   // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	 me2 *= 2.*LxJet::Nf;         // comes from flipped helicities and number of FS flavours
	 me2 *= hel_dim;
}
  


// __________________________________________________________________________________________________



void ME_2::calculate_me2_RgQQG() {   // the same as RgQb->QbG
    
  // crossing  
  assign(vtmp,k2);
  assigninv(k2,k4);
  assigninv(k4,vtmp);
    
         me2 = me2_RgGQQb_LLR();      // sum over helicities
         me2 += me2_RgGQQb_LRL();   
	 me2 /= (2.*M_PI)*2.*3.*8.;   // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	 me2 *= 2.;                   // comes from flipped helicities	 
}



// __________________________________________________________________________________________________



void ME_2::calculate_me2_RgQQG(double V[]) {   // the same as RgQb->QbG
  
  assign(vtmp,k2);
  assigninv(k2,k4);
  assigninv(k4,vtmp);
      
  int hel_dim = 2;  
  int hel_no = gen_int(V[0],hel_dim);
  
  switch (hel_no) {
    case 0:  
         me2 = me2_RgGQQb_LLR();
	 break;
    case 1:
         me2 = me2_RgGQQb_LRL();   
	 break;
  }
  
	 me2 /= (2.*M_PI)*2.*3.*8.;   // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	 me2 *= 2.;                   // comes from flipped helicities	 
	 me2 *= hel_dim;
}



// __________________________________________________________________________________________________



void ME_2::calculate_me2_RgGGG() { 

#ifdef RECURSION_2  
// ------ use Berends-Giele recursion --------  

k2.SetXYZT(-k2.X(),-k2.Y(),-k2.Z(),-k2.T());

assign(momenta[0],k2);
assign(momenta[1],k3);
assign(momenta[2],k4);

me2 = 0.;

  // loop over helicites
  for (int h=0; h<helicities.size(); h++) {
    
    // calculate polarization vectors
    for (int p=0; p<3; p++) polvecs[p].calc(helicities[h][p],momenta[p],n);
    
    // loop over color orderings
    for (int c=0; c<color_ords.size(); c++) {
      
       // make proper color order
       for (int u=1; u<4; u++) order[u-1] = polvecs[color_ords[c][u]-2];
       me2 += norm(amplitude(order));
    }}
    
         me2 /= (2.*M_PI)*2.*8.*8.;   // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	 me2 *= 2.;                   // comes from flipped helicities	 
	 me2 /= 2.;                   // symmetry factor
	 
	 
// ---------- end of Berends-Giele ----------------
#else
    
         me2 = me2_RgGGG_LLL();      // sum over helicities
         me2 += me2_RgGGG_LLR();
	 me2 += me2_RgGGG_LRL();
	 me2 /= (2.*M_PI)*2.*8.*8.;   // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	 me2 *= 2.;                   // comes from flipped helicities	 
	 me2 /= 2.;                   // symmetry factor

#endif
}



// __________________________________________________________________________________________________



void ME_2::calculate_me2_RgGGG(double V[]) { 
  
  int hel_dim = helicities.size();  
  int hel_no = gen_int(V[0],hel_dim);
  
#ifdef RECURSION_2  
// ------ use Berends-Giele recursion --------  

k2.SetXYZT(-k2.X(),-k2.Y(),-k2.Z(),-k2.T());

assign(momenta[0],k2);
assign(momenta[1],k3);
assign(momenta[2],k4);

me2 = 0.;

 // calculate polarization vectors
  for (int p=0; p<3; p++) polvecs[p].calc(helicities[hel_no][p],momenta[p],n);
    
    // loop over color orderings
    for (int c=0; c<color_ords.size(); c++) {
      
       // make proper color order
       for (int u=1; u<4; u++) order[u-1] = polvecs[color_ords[c][u]-2];
       me2 += norm(amplitude(order));
    }
    
         me2 /= (2.*M_PI)*2.*8.*8.;   // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	 me2 *= 2.;                   // comes from flipped helicities	 
	 me2 /= 2.;                   // symmetry factor
	 me2 *= hel_dim;

	 
// ---------- end of Berends-Giele ----------------
#else    

// ---------- use analytic formulae ---------------	 
	 
  switch (hel_no) {
    case 0:
         me2 = me2_RgGGG_LLL(); 
	 break;
    case 1:
         me2 = me2_RgGGG_LLR();
	 break;
    case 2:
	 me2 = me2_RgGGG_LRL();
	 break;
  }
  
	 me2 /= (2.*M_PI)*2.*8.*8.;   // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	 me2 *= 2.;                   // comes from flipped helicities	 
	 me2 /= 2.;                   // symmetry factor
	 me2 *= hel_dim;
#endif
}



// __________________________________________________________________________________________________


