//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "me_3.h"


// uncomment if the recursion relations for gluons are to be used
// they are a bit slower than analytic formulae
//#define RECURSION_3

//--------------------------------------------------------------------------------------------------
//------------------------------------------ ME_3 --------------------------------------------------
//--------------------------------------------------------------------------------------------------


ME_3::ME_3(const vector< vector<particles> > & state_space, int nproc) : ME_N(3)  {
  states = state_space.at(nproc);
  set_pointers(); }
 
 

// __________________________________________________________________________________________________



ME_3::ME_3(const vector<particles> & subproc) : ME_N(3)  {
  states = subproc;
  set_pointers();  }
  
 

// __________________________________________________________________________________________________



void ME_3::set_pointers() {
  
  // put the pointer of a function calculating value of ME to the functions_pt static table
  
  particles RgGGGG[] = {Rg,G,G,G,G};
  particles RgGQQbG[] = {Rg,G,Q,Qb,G};
  particles RgQQGG[] = {Rg,Q,Q,G,G};
  particles RgQQQQb[] = {Rg,Q,Q,Q,Qb};
  particles RgQQqqb[] = {Rg,Q,Q,q,qb};
  particles RgQbQbGG[] = {Rg,Qb,Qb,G,G};
  particles RgQbQbQQb[] = {Rg,Qb,Qb,Q,Qb};
  particles RgQbQbqqb[] = {Rg,Qb,Qb,q,qb};
  
  
  if ( is_permutation(states,vector_from_array(RgGGGG,5)) )
  { ME_3::calculate_pt = & ME_3::calculate_me2_RgGGGG;
    ME_3::calculate_hel_pt = & ME_3::calculate_me2_RgGGGG; 
    prepare_polvecs(); }
  
  else if ( is_permutation(states,vector_from_array(RgGQQbG,5)) )
   {ME_3::calculate_pt = & ME_3::calculate_me2_RgGQQbG;
    ME_3::calculate_hel_pt = & ME_3::calculate_me2_RgGQQbG;}
  
  else if ( is_permutation(states,vector_from_array(RgQQGG,5)) )
   { ME_3::calculate_pt = & ME_3::calculate_me2_RgQQGG;
     ME_3::calculate_hel_pt = & ME_3::calculate_me2_RgQQGG;}
  
  else if ( is_permutation(states,vector_from_array(RgQQQQb,5)) )
   { ME_3::calculate_pt = & ME_3::calculate_me2_RgQQQQb;
     ME_3::calculate_hel_pt = & ME_3::calculate_me2_RgQQQQb;}
  
  else if ( is_permutation(states,vector_from_array(RgQQqqb,5)) )
   { ME_3::calculate_pt = & ME_3::calculate_me2_RgQQqqb;
     ME_3::calculate_hel_pt = & ME_3::calculate_me2_RgQQqqb; }
    
  else if ( is_permutation(states,vector_from_array(RgQbQbGG,5)) )
   { ME_3::calculate_pt = & ME_3::calculate_me2_RgQbQbGG;
     ME_3::calculate_hel_pt = & ME_3::calculate_me2_RgQbQbGG; }
    
  else if ( is_permutation(states,vector_from_array(RgQbQbQQb,5)) )
  {  ME_3::calculate_pt = & ME_3::calculate_me2_RgQbQbQQb;
     ME_3::calculate_hel_pt = & ME_3::calculate_me2_RgQbQbQQb; }
    
  else if ( is_permutation(states,vector_from_array(RgQbQbqqb,5)) )
  {  ME_3::calculate_pt = & ME_3::calculate_me2_RgQbQbqqb;
     ME_3::calculate_hel_pt = & ME_3::calculate_me2_RgQbQbqqb; }
    
  else print_error(*LxJet::out_stream,"in ME_3::ME_3: process not implemented.");    
}



// __________________________________________________________________________________________________



void ME_3::fill_kinematics(const CONF_N & conf) {
  
    assign(k1,conf.get_k1());
    assign(k2,conf.get_k2());
    assign(k3,conf.get_ki(0));
    assign(k4,conf.get_ki(1));
    assign(k5,conf.get_ki(2));
    assign(e1,conf.get_e1());
    kTv = conf.get_kT(); 
    assign(n,k1);
    n -= kTv*e1;
}



// __________________________________________________________________________________________________



void ME_3::prepare_polvecs() {

 // prepare helicites
 helicity  hel_RLLL[] = {R,L,L,L};
 helicity  hel_RRLL[] = {R,R,L,L};
 helicity  hel_RLRL[] = {R,L,R,L};
 helicity  hel_RLLR[] = {R,L,L,R};
 helicity  hel_LRLL[] = {L,R,L,L};
 helicity  hel_LLRL[] = {L,L,R,L};
 helicity  hel_LLLR[] = {L,L,L,R};  

 helicities.clear();
 helicities.push_back(vector_from_array(hel_RLLL,4));
 helicities.push_back(vector_from_array(hel_RRLL,4));
 helicities.push_back(vector_from_array(hel_RLRL,4));
 helicities.push_back(vector_from_array(hel_RLLR,4));
 helicities.push_back(vector_from_array(hel_LRLL,4));
 helicities.push_back(vector_from_array(hel_LLRL,4));
 helicities.push_back(vector_from_array(hel_LLLR,4));
 
 // number of polarization vectors
 const int Np = 4;
 
 // prepare polarization vectors
 polvecs.clear();
 polvecs.resize(Np);
 momenta.clear();
 momenta.resize(Np);
 order.clear();
 order.resize(Np);

 // prepare color permutations
 color_ords = find_nonequiv_permutations(1,Np+1);
 
 // prepare array of the off-shell currents
    matrix_of_currents.clear();
    
     // construct trivial currents
     for (int i=0; i<Np; ++i) {
      tmp_VJ.push_back(tmp_J); }
      
     matrix_of_currents.push_back(tmp_VJ); 
     tmp_VJ.clear();
         
     // construct the rest
     for (int i=1; i<=(Np-2); i++) {
       for (int j=0; j<(Np-i); j++) {
	  tmp_VJ.push_back(tmp_J);  } 
        matrix_of_currents.push_back(tmp_VJ); 
	tmp_VJ.clear(); }
 
}



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgGGGG() {

#ifdef RECURSION_3  
// ------ use Berends-Giele recursion --------  

k2.SetXYZT(-k2.X(),-k2.Y(),-k2.Z(),-k2.T());


assign(momenta[0],k2);
assign(momenta[1],k3);
assign(momenta[2],k4);
assign(momenta[3],k5);


me2 = 0.;


  // loop over helicites
  for (int h=0; h<helicities.size(); h++) {
    
    // calculate polarization vectors
    for (int p=0; p<4; p++) polvecs[p].calc(helicities[h][p],momenta[p],n);
    
    // loop over color orderings
    for (int c=0; c<color_ords.size(); c++) {
      
       // make proper color order
       for (int u=1; u<5; u++) order[u-1] = polvecs[color_ords[c][u]-2];
       me2 += norm(amplitude(order));
    }
  }
      
      me2 *= 2.*27.*8.; // color factors
      me2 *= 2.;        // comes from the flipped helicities
      me2 /= (2.*M_PI)*2.*8.*8.; // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average

      //me2 = calculate_me2_ggggg_coll_ref();
	
      me2 /= 6.; // symmetry factor for identical particles

// ---------- end of Berends-Giele ----------------
#else

// ---------- use analytic formulae ---------------

            me2 = me2_RgGGGG_LLLL();    // sum over helicities
            me2 += me2_RgGGGG_LRLL();
	    me2 += me2_RgGGGG_LLRL();
	    me2 += me2_RgGGGG_LLLR();
	    me2 += me2_RgGGGG_RRLL();
	    me2 += me2_RgGGGG_RLRL();
	    me2 += me2_RgGGGG_RLLR();  
	    me2 *= 2.;                 // comes from flipped helicities
 	    me2 /= (2.*M_PI)*2.*8.*8.; // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average

            me2 /= 6.;                 // symmetry factor	
#endif    




}



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgGGGG(double V[]) {

  int hel_dim = helicities.size();  
  int hel_no = gen_int(V[0],hel_dim);

#ifdef RECURSION_3    
// ------ use Berends-Giele recursion --------  

k2.SetXYZT(-k2.X(),-k2.Y(),-k2.Z(),-k2.T());


assign(momenta[0],k2);
assign(momenta[1],k3);
assign(momenta[2],k4);
assign(momenta[3],k5);

me2 = 0.;

 // calculate polarization vectors
  for (int p=0; p<4; p++) polvecs[p].calc(helicities[hel_no][p],momenta[p],n);
    
    // loop over color orderings
    for (int c=0; c<color_ords.size(); c++) {
      
       // make proper color order
       for (int u=1; u<5; u++) order[u-1] = polvecs[color_ords[c][u]-2];
       me2 += norm(amplitude(order));
    }
  
      
      me2 *= 2.*27.*8.; // color factors
      me2 *= 2.;        // comes from the flipped helicities
      me2 /= (2.*M_PI)*2.*8.*8.; // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average	
      me2 /= 6.; // symmetry factor for identical particles
      me2 *= hel_dim;


// ---------- end of Berends-Giele ----------------
#else

// ---------- use analytic formulae ---------------

  switch (hel_no) {
    case 0:
       me2 = me2_RgGGGG_LLLL();    // sum over helicities
       break;
    case 1:
       me2 = me2_RgGGGG_LRLL();
       break;
    case 2:
       me2 = me2_RgGGGG_LLRL();
       break;
    case 3:
       me2 = me2_RgGGGG_LLLR();
       break;
    case 4:
       me2 = me2_RgGGGG_RRLL();
       break;
    case 5:
       me2 = me2_RgGGGG_RLRL();
       break;
    case 6:
       me2 = me2_RgGGGG_RLLR();  
       break;
  }
	
	    me2 *= 2.;                 // comes from flipped helicities
 	    me2 /= (2.*M_PI)*2.*8.*8.; // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
            me2 /= 6.;                 // symmetry factor	
            me2 *= hel_dim;
#endif	 
}



// __________________________________________________________________________________________________


 
void ME_3::calculate_me2_RgGQQbG() {
    
            me2 = me2_RgGQQbG_LLRL();  // sum over helicity
            me2 += me2_RgGQQbG_LRLL();
            me2 += me2_RgGQQbG_RLRL();
            me2 += me2_RgGQQbG_RRLL();
	    me2 *= 2.*LxJet::Nf;       // comes from flipped helicities and flavours
	    me2 /= (2.*M_PI)*2.*8.*8.; // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
}


// __________________________________________________________________________________________________


 
void ME_3::calculate_me2_RgGQQbG(double V[]) {
  
  int hel_dim = 4;
  int hel_no = gen_int(V[0],hel_dim);
  
  switch (hel_no) {
    case 0: 
        me2 = me2_RgGQQbG_LLRL();
     break;
    case 1: 
        me2 = me2_RgGQQbG_LRLL();
     break;
    case 2: 
        me2 = me2_RgGQQbG_RLRL();
     break;	
    case 3: 
        me2 = me2_RgGQQbG_RRLL();
     break;
  }
        me2 *= 2.*LxJet::Nf;       // comes from flipped helicities and flavours
	me2 /= (2.*M_PI)*2.*8.*8.; // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	me2 *= hel_dim;
}



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgQQGG() {
  
  //crossing
  assign(vtmp,k2);
  assigninv(k2,k4);
  assigninv(k4,vtmp);
    
            me2 = me2_RgGQQbG_LLRL();  // sum over helicites
            me2 += me2_RgGQQbG_LRLL();
            me2 += me2_RgGQQbG_RLRL();
            me2 += me2_RgGQQbG_RRLL();
	    me2 *= 2.;                 // comes from flipped helicities and flavours
	    me2 /= (2.*M_PI)*2.*8.*3.; // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	    
	    me2 /= 2.;                 // symmetry factor
}



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgQQGG(double V[]) {
  
  int hel_dim = 4;
  int hel_no = gen_int(V[0],hel_dim);
  
  //crossing
  assign(vtmp,k2);
  assigninv(k2,k4);
  assigninv(k4,vtmp);
  
  switch (hel_no) {
    case 0:
      me2 = me2_RgGQQbG_LLRL();
      break;
    case 1:
      me2 = me2_RgGQQbG_LRLL();
      break;
    case 2:
      me2 = me2_RgGQQbG_RLRL();
      break;
    case 3:
      me2 = me2_RgGQQbG_RRLL();
      break;
  }
  
  me2 *= 2.;                 // comes from flipped helicities and flavours
  me2 /= (2.*M_PI)*2.*8.*3.; // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
  me2 *= hel_dim;
  me2 /= 2.;                 // symmetry factor
}

    



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgQQqqb() {
	 
            me2 = me2_RgQQqqb_LLRL();  // sum over helicites
            me2 += me2_RgQQqqb_LLLR();
	    me2 *= 2.*(LxJet::Nf-1);   // comes from flipped helicities and flavours
	    me2 /= (2.*M_PI)*2.*3.*8.; // 2pi comes from continous polarization of 1 gluon
}



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgQQqqb(double V[]) {
  
  int hel_dim = 2;
  int hel_no = gen_int(V[0],hel_dim);
  
  switch (hel_no) {
    case 0:
       me2 = me2_RgQQqqb_LLRL();  // sum over helicites
       break;
    case 1:
       me2 = me2_RgQQqqb_LLLR();
       break;
  }
	    me2 *= 2.*(LxJet::Nf-1);   // comes from flipped helicities and flavours
	    me2 /= (2.*M_PI)*2.*3.*8.; // 2pi comes from continous polarization of 1 gluon
	    me2 *= hel_dim;
}



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgQbQbqqb() {
    
  // crossing
  assign(vtmp,k2);
  assigninv(k2,k3);
  assigninv(k3,vtmp);
	 
  calculate_me2_RgQQqqb();
}



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgQbQbqqb(double V[]) {
    
  // crossing
  assign(vtmp,k2);
  assigninv(k2,k3);
  assigninv(k3,vtmp);
	 
  calculate_me2_RgQQqqb(V);
}



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgQQQQb() {  
    
            me2 = me2_RgQQQQb_LLRL();   // sum over helicites
            me2 += me2_RgQQQQb_LLLR();
            me2 += me2_RgQQQQb_LRLL();
	    me2 *= 2.;                  // comes from flipped helicities
	    me2 /= (2.*M_PI)*2.*3.*8.;  // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	
            me2 /= 2.;                  // symmetry factor
}



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgQQQQb(double V[]) {  
  
  int hel_dim = 3;
  int hel_no = gen_int(V[0],hel_dim);
  
  switch (hel_no) {
    case 0:
      me2 = me2_RgQQQQb_LLRL();   // sum over helicites
      break;
    case 1:
      me2 = me2_RgQQQQb_LLLR();
      break;
    case 2:
      me2 = me2_RgQQQQb_LRLL();
      break;
  }
	    me2 *= 2.;                  // comes from flipped helicities
	    me2 /= (2.*M_PI)*2.*3.*8.;  // 2pi comes from continous polarization of the off-shell gluon; the rest is the spin and color average
	
            me2 /= 2.;                  // symmetry factor
            me2 *= hel_dim;
}



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgQbQbQQb() {  
  
  // crossing
  assign(vtmp,k2);
  assigninv(k2,k3);
  assigninv(k3,vtmp);
  
  calculate_me2_RgQQQQb();
}



// __________________________________________________________________________________________________



void ME_3::calculate_me2_RgQbQbQQb(double V[]) {  
  
  // crossing
  assign(vtmp,k2);
  assigninv(k2,k3);
  assigninv(k3,vtmp);
  
  calculate_me2_RgQQQQb(V);
}



// __________________________________________________________________________________________________



inline void ME_3::calculate_me2_RgQbQbGG() {
  
  calculate_me2_RgQQGG();
   
}



// __________________________________________________________________________________________________



inline void ME_3::calculate_me2_RgQbQbGG(double V[]) {
  
  calculate_me2_RgQQGG(V);
   
}



// __________________________________________________________________________________________________

