//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.



#include<cmath>
#include<algorithm>

#include "common.h"
#include "conf_N.h"
#include "me_N.h"


//--------------------------------------------------------------------------------------------------
//------------------------------------------ ME_N --------------------------------------------------
//--------------------------------------------------------------------------------------------------


COMPLEX ME_N::S(helicity hel_, const TLorentzVector & v1__, const TLorentzVector & v2__, const TLorentzVector & K__) {
     s1.calc(hel_,v1__);
     s2.calc(hel_,v2__);
     dirac_prod(cvec_tmp,s1,s2,ss);
    return cvec_tmp*K__; }



// __________________________________________________________________________________________________



bool ME_N::is_permutation( vector<particles> v1, vector<particles> v2) {
  sort(v1.begin(),v1.end());
  sort(v2.begin(),v2.end());
  return equal(v1.begin(),v1.end(),v2.begin());}



// __________________________________________________________________________________________________



void ME_N::remove_equiv(vector< vector<int> > & l) {
  int N = l.size();
  int i=0, j=0;
  vector<int> tmp;
  do {
    reverse(l.at(i).begin(),l.at(i).end());
    rotate(l.at(i).begin(),find(l.at(i).begin(),l.at(i).end(),l.at(N-1).at(0)),l.at(i).end());
     if (l.at(i)==l.at(N-1)) { 
       l.at(i).swap(l.at(j)); 
       l.pop_back(); N--; j++; i=j; }
     else i++;
  } while (j<N-1); }



// __________________________________________________________________________________________________



vector< vector< int > > ME_N::find_nonequiv_permutations(int n_, int m_) {
 vector< int >::iterator it;
 vector< vector< int > > perms;
 vector< int > vec;
 for (int i=n_; i<=m_; i++) vec.push_back(i);
 
 do {
    perms.push_back(vec);
  } while ( std::next_permutation(vec.begin(),vec.end()) );  
 for (int i=0; i<perms.size(); i++) {
  it = find(perms[i].begin(),perms[i].end(),vec[0]);
  rotate(perms[i].begin(),it,perms[i].end()); }
  sort(perms.begin(),perms.end());
 vector< vector< int > >::iterator itt;
 itt = unique(perms.begin(),perms.end());
 perms.resize(distance(perms.begin(),itt)); 
 remove_equiv(perms);
  for (int i=0; i<perms.size(); i++) {
  it = find(perms[i].begin(),perms[i].end(),vec[0]);
  rotate(perms[i].begin(),it,perms[i].end()); }
 return perms;
  }



// __________________________________________________________________________________________________



COMPLEX ME_N::amplitude(const vector<POLVEC> & e_) { 
  
     int N = e_.size();
     
     // construct trivial currents
     for (int i=0; i<N; i++) {
      matrix_of_currents[0][i].mom = e_[i].get_mom();
      matrix_of_currents[0][i].curr = e_[i].get_polvec(); }
         
     // construct the rest
     for (int i=1; i<=(N-2); i++) {
       for (int j=0; j<(N-i); j++) {
	  recursion_calc(tmp_J,j,j+i);	 
	  propagator_axial(tmp_J.mom,tmp_J.curr);	  
	  assign(matrix_of_currents[i][j].mom,tmp_J.mom);
	  matrix_of_currents[i][j].curr = tmp_J.curr; }}
	
      recursion_calc(tmp_J,0,N-1);
      //check_current_conserv();
      
     //calculate waste saving gauge invariance
	waste = -kTv/pow(-sqrt(2.),N-1);
	for (int i=0; i<N; ++i) waste *= e_[i].get_polvec()*n;
	for (int i=0; i<(N-1); ++i) waste /= -(JJ(0,i).mom*n);

	return  -Ii*(e1*tmp_J.curr)+waste;    }
