//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.



#include <algorithm>
#include <fstream>

#include "pdf.h"




//--------------------------------------------------------------------------------------------------
//-------------------------------------------- PDF -------------------------------------------------
//--------------------------------------------------------------------------------------------------


void PDF::load_pdf_table(const char* file_name)
{
  
  ifstream in(file_name);
  
  double x,y,f;
  
 vector<double> x_tab_, y_tab_, f_tab_;
  
  if ( in.good() == true ) {   
   do  {
      in >> x >> y >> f;
      x_tab_.push_back(rescale_X(x));
      y_tab_.push_back(rescale_Y(y));
      f_tab_.push_back(rescale_val(f,x,y));
      if (in.eof()) break; }   
    while (1); }
  else 
    cout << "ERROR: PDF::load_pdf_table: couldn't open the file " << file_name << endl;
  
  // remove the last doubled element
  x_tab_.pop_back();
  y_tab_.pop_back();
//  f_tab_.pop_back();
  
  vector<double> x_tab__(x_tab_.size(),0.);
  vector<double> y_tab__(y_tab_.size(),0.);
  sort(y_tab_.begin(),y_tab_.end());
  unique_copy(x_tab_.begin(),x_tab_.end(),x_tab__.begin());
  unique_copy(y_tab_.begin(),y_tab_.end(),y_tab__.begin());
  
  // remove zeros
  do x_tab__.pop_back(); while (x_tab__.back()==0.);
  do  y_tab__.pop_back(); while (y_tab__.back()==0.);
  
  int Nx = x_tab__.size();
  int Ny = y_tab__.size();

  f_tab = new double[Nx*Ny];

  int k = 0;
  for (int i=0; i<Nx; i++) for (int j=0; j<Ny; j++) {
   f_tab[k] = f_tab_.at(k);
    k++; }

  // --- set interpolation object ---
  xseq = new VSeq_t;
  yseq = new VSeq_t;
  xseq->Assign(x_tab__);
  yseq->Assign(y_tab__);
  interpol = new Interpolator2_t(*xseq,*yseq,f_tab,int_ord_x,int_ord_y); 
  // -------------------------------------

  // set the maximal and minimal values of x and kt (or mu2)
  xgrid_max = *max_element(x_tab__.begin(),x_tab__.end());
  xgrid_min = *min_element(x_tab__.begin(),x_tab__.end());
  ygrid_max = *max_element(y_tab__.begin(),y_tab__.end());
  ygrid_min = *min_element(y_tab__.begin(),y_tab__.end()); 
  
  using_grid = true;

  in.close();
}



//--------------------------------------------------------------------------------------------------
//----------------------------------------- ALPHAS -------------------------------------------------
//--------------------------------------------------------------------------------------------------

