//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "pdfprod.h"




//--------------------------------------------------------------------------------------------------
//----------------------------------------- PDF ----------------------------------------------------
//--------------------------------------------------------------------------------------------------

PDFPROD::PDFPROD(const vector<particles> & states_,PDF &hadron_A_,PDF &hadron_B_) {
   
 partonA = states_[0];
 partonB = states_[1];
 
 double _xAmin_ = LxJet::xA_min;
 double _xAmax_ = LxJet::xA_max;
 double _xBmin_ = LxJet::xB_min;
 double _xBmax_ = LxJet::xB_max;
 double _kTmin_ = LxJet::kT_min;
 double _kTmax_ = LxJet::kT_max;
 
 
 // check if hadron_A is collinear
 bool is_A_coll = hadron_A_.is_collinear();
 
 // check if hadron_B is collinear
 bool is_B_coll = hadron_B_.is_collinear();
  
 if (!is_A_coll && !is_B_coll) print_error(*LxJet::out_stream,"in PDFPROD::PDFPROD: only collisions with single off-shell gluon are currently implemented.");
  // if there is at least one collinear parton
  else {  
    
    if ( !(partonA!=Rg || partonB!=Rg) ) print_error(*LxJet::out_stream,"in PDFPROD::PDFPROD: at least one initial state parton should be collinear.");
     else 
       if (is_A_coll && is_B_coll) PDFPROD::calculate_pdfs_pt = & PDFPROD::calculate_coll_pdfs;
        else PDFPROD::calculate_pdfs_pt = & PDFPROD::calculate_mixed_pdfs;
      
    
  }
 
 hadronA_pt = &hadron_A_;
 hadronB_pt = &hadron_B_;
 
 switch(partonA) {
   case Rg:
     PDFPROD::pdf_A_pt = & PDFPROD::pdf_G;
     break;
   case G:
     PDFPROD::pdf_A_pt = & PDFPROD::pdf_G;
     break;
   case Q:
     PDFPROD::pdf_A_pt = & PDFPROD::pdf_Q;
     break;
   case Qb:
     PDFPROD::pdf_A_pt = & PDFPROD::pdf_Qb;
     break;
   default:
     print_error(*LxJet::out_stream,"in PDFPROD::PDFPROD: wrong initial state."); }
 
 
 switch(partonB) {
   case Rg:
     PDFPROD::pdf_B_pt = & PDFPROD::pdf_G;
     break;
   case G:
     PDFPROD::pdf_B_pt = & PDFPROD::pdf_G;
     break;
   case Q:
     PDFPROD::pdf_B_pt = & PDFPROD::pdf_Q;
     break;
   case Qb:
     PDFPROD::pdf_B_pt = & PDFPROD::pdf_Qb;
     break;
   default:
     print_error(*LxJet::out_stream,"in PDFPROD::PDFPROD: wrong initial state."); }
     
     
   // adjust the boundaries of x and kt
   
   _xAmin_ = hadron_A_.get_xgrid_min();
   _xAmax_ = hadron_A_.get_xgrid_max();
   
   _xBmin_ = hadron_B_.get_xgrid_min();
   _xBmax_ = hadron_B_.get_xgrid_max();
   
   if (!is_A_coll) {
   _kTmin_ = hadron_A_.get_ygrid_min();
   _kTmax_ = hadron_A_.get_ygrid_max(); }
    else {
     _kTmin_ = 0.;
     _kTmax_ = 0.; }
   
   if (!is_B_coll) print_error(*LxJet::out_stream,"in PDFPROD::PDFPROD: two off-shell gluons are not implemented.");
       
    
   if ( LxJet::xA_min < _xAmin_ ) {
     no_kin_modif++;
     LxJet::xA_min = _xAmin_;
     print_message(*LxJet::out_stream,"Decreased the value of xA_min according to the PDF grid");
     print_message(*LxJet::out_stream,(string)"New value: xA_min = "<<_xAmin_);
     
  }
     
   if ( LxJet::xA_max > _xAmax_ ) {
     no_kin_modif++;
     LxJet::xA_max = _xAmax_;
     print_message(*LxJet::out_stream,"Decreased the value of xA_max according to the PDF grid");
     print_message(*LxJet::out_stream,(string)"New value: xA_max = "<<_xAmax_);
  }
   
   if ( LxJet::xB_min < _xBmin_ ) {
     no_kin_modif++;
     LxJet::xB_min = _xBmin_;
     print_message(*LxJet::out_stream,"Decreased the value of xB_min according to the PDF grid"); 
     print_message(*LxJet::out_stream,(string)"New value: xB_min = "<<_xBmin_);
  }
     
   if ( LxJet::xB_max > _xBmax_ ) {
     no_kin_modif++;
     LxJet::xB_max = _xBmax_;
     print_message(*LxJet::out_stream,"Decreased the value of xB_max according to the PDF grid"); 
     print_message(*LxJet::out_stream,(string)"New value: xB_max = "<<_xBmax_);
  }
   
   if ( LxJet::kT_min < _kTmin_ ) {
     no_kin_modif++;
     LxJet::kT_min = _kTmin_;
     print_message(*LxJet::out_stream,"Decreased the value of kT_min according to the PDF grid");
     print_message(*LxJet::out_stream,(string)"New value: kT_min = "<<_kTmin_);
     
  }
     
   if ( LxJet::kT_max > _kTmax_ ) {
     no_kin_modif++;
     LxJet::kT_max = _kTmax_;
     print_message(*LxJet::out_stream,"Decreased the value of kT_max according to the PDF grid"); 
     print_message(*LxJet::out_stream,(string)"New value: kT_max = "<<_kTmax_);
  }

if (no_kin_modif>6) // 6 = xAmin,xAmax,xBmin,xBmax,kTmin,kTmax
  print_message(*LxJet::out_stream,"WARNING: objects of PDFPROD for various partons do not use the same PDFs!");
}



// __________________________________________________________________________________________________



/// static member to count the number of changes in kinematic limits
int PDFPROD::no_kin_modif=0;
