//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.





#include "userfun.h"




USER_CLASS::USER_CLASS():optimized_basket(0) {
   
  // book process spaces for use in derived classes
   dijets = new PROCSPACE(2);
   trijets = new PROCSPACE(3);
  
  // set collinear PDF
   collinear_proton = new PDF_cteq;
   
  // set alpha strong
   alphaqcd = new ALPHAS_cteq; 
   
   //------ for printing trial events to a file ---
       //count_ev = 0;
       //file.open("events.txt"); 
   //----------------------------------------------
}
  



void USER_CLASS::book() {

  //count_ev = 0;
  //     file.open("events.txt"); 
  
  root_ev = new LXEVENT;
  string tree_name = get_alias()+string("tree");
  string file_name = get_alias()+string(".root");
  root_file = new TFile(file_name.c_str(),"recreate");
  root_file->SetCompressionLevel(2);
  root_tree = new TTree(tree_name.c_str(),tree_name.c_str());
  root_tree->SetAutoSave(100000000);
  root_tree->Branch("events","LXEVENT",&root_ev,64000);
  root_tree->Branch("CM_energy",&LxJet::CM_energy,"CM_energy/D");  
  root_tree->Branch("xA_min",&LxJet::xA_min,"xA_min/D");
  root_tree->Branch("xB_min",&LxJet::xB_min,"xB_min/D");
  root_tree->Branch("xA_max",&LxJet::xA_max,"xA_max/D");
  root_tree->Branch("xB_max",&LxJet::xB_max,"xB_max/D");
}



void USER_CLASS::fill()
{
 root_ev->set_in_states(event[0],event[1]);
 root_ev->set_out_states(jets);
 root_ev->set_weight(get_weight());
 root_ev->set_flavours(get_state_space(get_event_number())); // needed for parton shower 
 root_tree->Fill();
 root_ev->Clear();         
}

void USER_CLASS::save() {
  root_ev->set_normalization(get_integral());
    if (root_tree->GetEntries() > 1000000 && !optimized_basket) {
      root_tree->OptimizeBaskets();
      optimized_basket = true; }
      root_tree->Write();       
} 

void USER_CLASS::save_trial_events(int start_, int stop_) {     
   if (count_ev>start_ && count_ev<stop_) {
     vector<TLorentzVector> dump;
     dump.push_back(event[0]);
     dump.push_back(event[1]);
     dump.push_back(jets[0]);
     dump.push_back(jets[1]);
     dump.push_back(jets[2]);
     print_3D_event(file,dump); }
   count_ev++;
}
  

