//    Copyright (C) 2013 Piotr Kotko
//
//    This file is part of LxJet
//
//    LxJet is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.



#include "xsec.h"
#include "conf_2.h"
#include "conf_3.h"
#include "me_2.h"
#include "me_3.h"

#include "TMath.h"
#include "Math/SpecFuncMathMore.h"

//--------------------------------------------------------------------------------------------------
//---------------------------------------------- XSEC ----------------------------------------------
//--------------------------------------------------------------------------------------------------

#include <fstream>

ofstream file;

XSEC::XSEC(LXJET* &lxjet):lxjet_pt(lxjet) {
  
  file.open("pdf_dump.txt");
  
  Ntotproc = lxjet->get_state_space().size();
  
  // WARNING!! The PDFs have to be booked prior to CONF class !!!!!!!!!!
  
      switch(lxjet->get_Nfs()) {
		
	case 2:
	 	 
         for (int i=0; i< Ntotproc; i++) {
	  mels.push_back(new ME_2(lxjet->get_state_space(),i));   
	  pdfs.push_back(new PDFPROD(lxjet->get_state_space(i),*lxjet->get_hadron_A(),*lxjet->get_hadron_B())); 
	}
	
	conf = new CONF_2;
	  
	 break;
	
	case 3:
	
	 	 	 
         for (int i=0; i< Ntotproc; i++) {
	  mels.push_back(new ME_3(lxjet->get_state_space(),i));
	  pdfs.push_back(new PDFPROD(lxjet->get_state_space(i),*lxjet->get_hadron_A(),*lxjet->get_hadron_B())); 
	}
	
	conf = new CONF_3;
	  
	 break;
      }
      
       // switch of symmterization if hadrons are not symmetric      
      if ( lxjet->are_symmetrized() )  conf->set_symmetrize(1);
        else 
	  conf->set_symmetrize(0);
}



// __________________________________________________________________________________________________



XSEC::~XSEC() {
      	
         delete conf;
	 
         for (int i=0; i< Ntotproc; i++) {
	   
	  delete mels[i];
	  delete pdfs[i];
	 }
}



// __________________________________________________________________________________________________



double XSEC::Density(int ndim, double V[]) {
 
  // Generate the event number
  event_no = gen_int(V[lxjet_pt->get_Ndim()-2],Ntotproc);
    
  // Generate the kinematic configuration
  conf->generate(V);
  
  retain_event = !conf->get_reject();
  
  if (retain_event) retain_event *= !lxjet_pt->basic_cuts(*conf);
   else return 0.;
   
  if (retain_event)  {

   // mels[event_no]->calculate_me2(*conf);  // this is used when explicit sum over polarizations
    mels[event_no]->calculate_me2(*conf,V+(lxjet_pt->get_Ndim()-1)); // MC sum over polarizations
          
    pdfs[event_no]->calculate_pdfs(conf->get_xA(),conf->get_xB(),lxjet_pt->get_mu_fact(),conf->get_kT());
        
    weight = lxjet_pt->get_mult_fact()
	     *Ntotproc       // summation over events
             *pow(4.*M_PI*lxjet_pt->get_alphas(),mels[event_no]->get_alphas_pow())
             *pdfs[event_no]->get_pdfs()
             *mels[event_no]->get_me2()
	     *conf->get_phase_space_factor()
	     /(2.*pow2(LxJet::CM_energy)*conf->get_xA()*conf->get_xB());
	
   wt_sign = signums(weight);
	       
   return abs(weight);
       
  }
  
  else return 0.;


}

  
